@@ -18,7 +18,7 @@
 
 #include "amarokconfig.h"
 #include "core/support/Debug.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
 #include "SqlCollection.h"
 
 #include <QDateTime>
