@@ -20,6 +20,8 @@
 #define SQLMETA_H
 
 #include "core/meta/Meta.h"
+#include "core/meta/Statistics.h"
+#include "core/meta/TrackEditor.h"
 #include "core/meta/support/MetaConstants.h"
 #include "amarok_sqlcollection_export.h"
 #include "FileType.h"
@@ -58,7 +60,7 @@ namespace Meta
 
     The whole class should be thread save.
 */
-class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
+class AMAROK_SQLCOLLECTION_EXPORT SqlTrack : public Track, public Statistics, public TrackEditor
 {
     public:
         /** Creates a new SqlTrack without.
@@ -72,86 +74,69 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
         SqlTrack( Collections::SqlCollection *collection, const QStringList &queryResult );
         ~ SqlTrack();
 
-        /** returns the title of this track as stored in the database **/
         virtual QString name() const;
-        /** returns the title of the track if existing in the database,
-            a value deduced from the file name otherwise */
         virtual QString prettyName() const;
-        /** returns "[artist] - [title]" if both are stored in the database,
-            a value deduced from the file name otherwise */
-        virtual QString fullPrettyName() const;
-
-        /** returns the KUrl object describing the position of the track */
         virtual KUrl playableUrl() const;
-
-        /** returns a string describing the position of the track; same as url() */
         virtual QString prettyUrl() const;
-
-        /** returns a string describing the position of the track */
         virtual QString uidUrl() const;
-        virtual void setUidUrl( const QString &uid );
-
-        /** true if there is a collection and the file exists on disk */
-        virtual bool isPlayable() const;
-        /** true if there is a collection, the file exists on disk and is writable */
-        virtual bool isEditable() const;
+        virtual QString notPlayableReason() const;
 
         virtual Meta::AlbumPtr album() const;
-        virtual void setAlbum( const QString &newAlbum );
-        virtual void setAlbum( int albumId );
-
-        virtual void setAlbumArtist( const QString &newAlbumArtist );
-
-        virtual void setArtist( const QString &newArtist );
         virtual Meta::ArtistPtr artist() const;
         virtual Meta::ComposerPtr composer() const;
-        virtual void setComposer( const QString &newComposer );
         virtual Meta::YearPtr year() const;
-        virtual void setYear( int newYear );
         virtual Meta::GenrePtr genre() const;
-        virtual void setGenre( const QString &newGenre );
 
         virtual QString type() const;
-        virtual void setType( Amarok::FileType newType );
-
-        virtual void setTitle( const QString &newTitle );
-
-        virtual void setUrl( int deviceId, const QString &rpath, int directoryId );
-
         virtual qreal bpm() const;
+        virtual QString comment() const;
+        virtual qint64 length() const;
+        virtual int filesize() const;
+        virtual int sampleRate() const;
+        virtual int bitrate() const;
+        virtual QDateTime createDate() const;
+        virtual QDateTime modifyDate() const;
+        virtual int trackNumber() const;
+        virtual int discNumber() const;
+        virtual qreal replayGain( Meta::ReplayGainTag mode ) const;
+
+        virtual bool inCollection() const;
+        virtual Collections::Collection* collection() const;
+
+        virtual QString cachedLyrics() const;
+        virtual void setCachedLyrics( const QString &lyrics );
+
+        virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const;
+        virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type );
+
+        virtual void addLabel( const QString &label );
+        virtual void addLabel( const Meta::LabelPtr &label );
+        virtual void removeLabel( const Meta::LabelPtr &label );
+        virtual Meta::LabelList labels() const;
+
+        virtual TrackEditorPtr editor();
+        virtual StatisticsPtr statistics();
+
+        // Meta::TrackEditor methods:
+        virtual void setAlbum( const QString &newAlbum );
+        virtual void setAlbumArtist( const QString &newAlbumArtist );
+        virtual void setArtist( const QString &newArtist );
+        virtual void setComposer( const QString &newComposer );
+        virtual void setGenre( const QString &newGenre );
+        virtual void setYear( int newYear );
+        virtual void setTitle( const QString &newTitle );
+        virtual void setComment( const QString &newComment );
+        virtual void setTrackNumber( int newTrackNumber );
+        virtual void setDiscNumber( int newDiscNumber );
         virtual void setBpm( const qreal newBpm );
 
-        virtual QString comment() const;
-        virtual void setComment( const QString &newComment );
-
+        // Meta::Statistics methods:
         virtual double score() const;
         virtual void setScore( double newScore );
 
         virtual int rating() const;
         virtual void setRating( int newRating );
 
-        virtual qint64 length() const;
-        virtual void setLength( qint64 newLength );
-
-        virtual int filesize() const;
-
-        virtual int sampleRate() const;
-        virtual void setSampleRate( int newSampleRate );
-
-        virtual int bitrate() const;
-        virtual void setBitrate( int newBitrate );
-
-        virtual QDateTime createDate() const;
-
-        virtual QDateTime modifyDate() const;
-        virtual void setModifyDate( const QDateTime &newTime );
-
-        virtual int trackNumber() const;
-        virtual void setTrackNumber( int newTrackNumber );
-
-        virtual int discNumber() const;
-        virtual void setDiscNumber( int newDiscNumber );
-
         virtual QDateTime firstPlayed() const;
         virtual void setFirstPlayed( const QDateTime &newTime );
 
@@ -161,10 +146,23 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
         virtual int playCount() const;
         virtual void setPlayCount( const int newCount );
 
-        virtual qreal replayGain( Meta::ReplayGainTag mode ) const;
-        virtual void setReplayGain( Meta::ReplayGainTag mode, qreal value );
-        virtual void beginMetaDataUpdate();
-        virtual void endMetaDataUpdate();
+        // combined Meta::Statistics and Meta::TrackEditor methods:
+        virtual void beginUpdate();
+        virtual void endUpdate();
+
+        // SqlTrack specific methods
+        /** true if there is a collection, the file exists on disk and is writable */
+        bool isEditable() const;
+
+        void setUidUrl( const QString &uid );
+        void setAlbum( int albumId );
+        void setType( Amarok::FileType newType );
+        void setLength( qint64 newLength );
+        void setSampleRate( int newSampleRate );
+        void setUrl( int deviceId, const QString &rpath, int directoryId );
+        void setBitrate( int newBitrate );
+        void setModifyDate( const QDateTime &newTime );
+        void setReplayGain( Meta::ReplayGainTag mode, qreal value );
 
         /** Enables or disables writing changes to the file.
          *  This function can be useful when changes are imported from the file.
@@ -173,25 +171,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
         virtual void setWriteFile( const bool enable )
         { m_writeFile = enable; }
 
-        virtual void finishedPlaying( double playedFraction );
-
-        virtual bool inCollection() const;
-        virtual Collections::Collection* collection() const;
-
-        virtual QString cachedLyrics() const;
-        virtual void setCachedLyrics( const QString &lyrics );
-
-        virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const;
-
-        virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type );
-
-        virtual void addLabel( const QString &label );
-        virtual void addLabel( const Meta::LabelPtr &label );
-        virtual void removeLabel( const Meta::LabelPtr &label );
-        virtual Meta::LabelList labels() const;
-
-        // SqlTrack specific methods
-
         int id() const;
         int urlId() const;
         Collections::SqlCollection* sqlCollection() const { return m_collection; }
@@ -214,7 +193,7 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
             returnIndex_urlId = 0,
             returnIndex_urlDeviceId = 1,
             returnIndex_urlRPath = 2,
-            returnIndex_urlUid = 3,
+            returnIndex_urlUid = 4,
             returnIndex_trackId = 5
         };
 
@@ -229,19 +208,18 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
 
 
     protected:
-        /** Will commit all changes in m_cache.
-         *  commitMetaDataChanges will do three things:<br>
+        /**
+         * Will commit all changes in m_cache if m_batch == 0. Must be called with m_lock
+         * locked for writing.
+         *
+         *  commitIfInNonBatchUpdate() will do three things:
          *  1. It will update the member variables.
          *  2. It will call all write methods
          *  3. It will notify all observers and the collection about the changes.
          */
-        void commitMetaDataChanges();
-        void commitMetaDataChanges( qint64 field, const QVariant &value )
-        {
-            m_cache.insert( field, value );
-            if( !m_batchUpdate )
-                commitMetaDataChanges();
-        }
+        void commitIfInNonBatchUpdate( qint64 field, const QVariant &value );
+        void commitIfInNonBatchUpdate();
+
         void updatePlaylistsToDb( const FieldHash &fields, const QString &oldUid );
         void updateEmbeddedCoversToDb( const FieldHash &fields, const QString &oldUid );
 
@@ -293,7 +271,11 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
 
         Amarok::FileType m_filetype;
 
-        bool m_batchUpdate;
+        /**
+         * Number of current batch operations started by @see beginUpdate() and not
+         * yet ended by @see endUpdate(). Must only be accessed with m_lock held.
+         */
+        int m_batchUpdate;
         bool m_writeFile;
         bool m_writeAllStatisticsFields;
         FieldHash m_cache;
@@ -313,7 +295,7 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
         friend class ::TrackStatisticsTableCommitter;
 };
 
-class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlArtist : public Meta::Artist
+class AMAROK_SQLCOLLECTION_EXPORT SqlArtist : public Meta::Artist
 {
     public:
         SqlArtist( Collections::SqlCollection* collection, int id, const QString &name );
@@ -348,7 +330,7 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlArtist : public Meta::Artist
     Note: The album without name is special. It will always be a compilation
     and never have a picture.
 */
-class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlAlbum : public Meta::Album
+class AMAROK_SQLCOLLECTION_EXPORT SqlAlbum : public Meta::Album
 {
     public:
         SqlAlbum( Collections::SqlCollection* collection, int id, const QString &name, int artist );
@@ -475,7 +457,7 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlAlbum : public Meta::Album
         friend class ::SqlScanResultProcessor; // needs to set images directly
 };
 
-class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlComposer : public Meta::Composer
+class AMAROK_SQLCOLLECTION_EXPORT SqlComposer : public Meta::Composer
 {
     public:
         SqlComposer( Collections::SqlCollection* collection, int id, const QString &name );
@@ -533,7 +515,7 @@ class SqlGenre : public Meta::Genre
         friend class Meta::SqlTrack; // needs to call notifyObservers
 };
 
-class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlYear : public Meta::Year
+class AMAROK_SQLCOLLECTION_EXPORT SqlYear : public Meta::Year
 {
     public:
         SqlYear( Collections::SqlCollection* collection, int id, int year );
@@ -564,7 +546,7 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlYear : public Meta::Year
         friend class Meta::SqlTrack; // needs to call notifyObservers
 };
 
-class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlLabel : public Meta::Label
+class AMAROK_SQLCOLLECTION_EXPORT SqlLabel : public Meta::Label
 {
 public:
     SqlLabel( Collections::SqlCollection *collection, int id, const QString &name );
