@@ -15,15 +15,15 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
-#include "SqlQueryMaker.h"
-
 #define DEBUG_PREFIX "SqlQueryMaker"
 
-#include "core/support/Debug.h"
+#include "SqlQueryMaker.h"
 
 #include "SqlCollection.h"
 #include "SqlQueryMakerInternal.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
+#include "core/support/Debug.h"
+#include "core-impl/collections/db/MountPointManager.h"
 
 #include <QWeakPointer>
 #include <QStack>
@@ -82,20 +82,15 @@ struct SqlQueryMaker::Private
     QString queryMatch;
     QString queryFilter;
     QString queryOrderBy;
-    bool includedBuilder;
-    bool collectionRestriction;
-    bool resultAsDataPtrs;
     bool withoutDuplicates;
     int maxResultSize;
     AlbumQueryMode albumMode;
-    ArtistQueryMode artistMode;
     LabelQueryMode labelMode;
     SqlWorkerThread *worker;
 
     QStack<bool> andStack;
 
     QStringList blockingCustomData;
-    Meta::DataList blockingData;
     Meta::TrackList blockingTracks;
     Meta::AlbumList blockingAlbums;
     Meta::ArtistList blockingArtists;
@@ -113,15 +108,11 @@ SqlQueryMaker::SqlQueryMaker( SqlCollection* collection )
     , m_collection( collection )
     , d( new Private )
 {
-    d->includedBuilder = true;
-    d->collectionRestriction = false;
     d->worker = 0;
     d->queryType = QueryMaker::None;
     d->linkedTables = 0;
-    d->resultAsDataPtrs = false;
     d->withoutDuplicates = false;
     d->albumMode = AllAlbums;
-    d->artistMode = TrackArtists;
     d->labelMode = QueryMaker::NoConstraint;
     d->maxResultSize = -1;
     d->andStack.clear();
@@ -135,7 +126,8 @@ SqlQueryMaker::~SqlQueryMaker()
 {
     disconnect();
     abortQuery();
-    // TODO: check d->worker. Shouldn't it be deleted?
+    if( d->worker )
+        d->worker->deleteLater();
     delete d;
 }
 
@@ -151,18 +143,6 @@ SqlQueryMaker::abortQuery()
     }
 }
 
-QueryMaker*
-SqlQueryMaker::setReturnResultAsDataPtrs( bool resultAsDataPtrs )
-{
-    // we need the unchanged resulttype in the blocking result methods so prevent
-    // reseting result type without reseting the QM
-    if ( d->blocking && d->used )
-        return this;
-
-    d->resultAsDataPtrs = resultAsDataPtrs;
-    return this;
-}
-
 void
 SqlQueryMaker::run()
 {
@@ -182,35 +162,31 @@ SqlQueryMaker::run()
         SqlQueryMakerInternal *qmi = new SqlQueryMakerInternal( m_collection );
         qmi->setQuery( query() );
         qmi->setQueryType( d->queryType );
-        qmi->setResultAsDataPtrs( d->resultAsDataPtrs );
-
 
         if ( !d->blocking )
         {
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::AlbumList)), SIGNAL(newResultReady(QString,Meta::AlbumList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::ArtistList)), SIGNAL(newResultReady(QString,Meta::ArtistList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::GenreList)), SIGNAL(newResultReady(QString,Meta::GenreList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::ComposerList)), SIGNAL(newResultReady(QString,Meta::ComposerList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::YearList)), SIGNAL(newResultReady(QString,Meta::YearList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::TrackList)), SIGNAL(newResultReady(QString,Meta::TrackList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::DataList)), SIGNAL(newResultReady(QString,Meta::DataList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,QStringList)), SIGNAL(newResultReady(QString,QStringList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::LabelList)), SIGNAL(newResultReady(QString,Meta::LabelList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::AlbumList)),    SIGNAL(newResultReady(Meta::AlbumList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::ArtistList)),   SIGNAL(newResultReady(Meta::ArtistList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::GenreList)),    SIGNAL(newResultReady(Meta::GenreList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::ComposerList)), SIGNAL(newResultReady(Meta::ComposerList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::YearList)),     SIGNAL(newResultReady(Meta::YearList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::TrackList)),    SIGNAL(newResultReady(Meta::TrackList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(QStringList)),        SIGNAL(newResultReady(QStringList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::LabelList)),    SIGNAL(newResultReady(Meta::LabelList)), Qt::DirectConnection );
             d->worker = new SqlWorkerThread( qmi );
-            connect( d->worker, SIGNAL( done( ThreadWeaver::Job* ) ), SLOT( done( ThreadWeaver::Job* ) ) );
+            connect( d->worker, SIGNAL(done(ThreadWeaver::Job*)), SLOT(done(ThreadWeaver::Job*)) );
             ThreadWeaver::Weaver::instance()->enqueue( d->worker );
         }
         else //use it blocking
         {
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::AlbumList)), SLOT(blockingNewResultReady(QString,Meta::AlbumList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::ArtistList)), SLOT(blockingNewResultReady(QString,Meta::ArtistList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::GenreList)), SLOT(blockingNewResultReady(QString,Meta::GenreList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::ComposerList)), SLOT(blockingNewResultReady(QString,Meta::ComposerList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::YearList)), SLOT(blockingNewResultReady(QString,Meta::YearList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::TrackList)), SLOT(blockingNewResultReady(QString,Meta::TrackList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::DataList)), SLOT(blockingNewResultReady(QString,Meta::DataList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,QStringList)), SLOT(blockingNewResultReady(QString,QStringList)), Qt::DirectConnection );
-            connect( qmi, SIGNAL(newResultReady(QString,Meta::LabelList)), SLOT(blockingNewResultReady(QString,Meta::LabelList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::AlbumList)),    SLOT(blockingNewResultReady(Meta::AlbumList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::ArtistList)),   SLOT(blockingNewResultReady(Meta::ArtistList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::GenreList)),    SLOT(blockingNewResultReady(Meta::GenreList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::ComposerList)), SLOT(blockingNewResultReady(Meta::ComposerList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::YearList)),     SLOT(blockingNewResultReady(Meta::YearList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::TrackList)),    SLOT(blockingNewResultReady(Meta::TrackList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(QStringList)),        SLOT(blockingNewResultReady(QStringList)), Qt::DirectConnection );
+            connect( qmi, SIGNAL(newResultReady(Meta::LabelList)),    SLOT(blockingNewResultReady(Meta::LabelList)), Qt::DirectConnection );
             qmi->run();
             delete qmi;
         }
@@ -221,9 +197,8 @@ SqlQueryMaker::run()
 void
 SqlQueryMaker::done( ThreadWeaver::Job *job )
 {
-    ThreadWeaver::Weaver::instance()->dequeue( job );
     job->deleteLater();
-    d->worker = 0;
+    d->worker = 0; // d->worker *is* the job, prevent stale pointer
     emit queryDone();
 }
 
@@ -336,28 +311,6 @@ SqlQueryMaker::setQueryType( QueryType type )
     return this;
 }
 
-QueryMaker*
-SqlQueryMaker::includeCollection( const QString &collectionId )
-{
-    if( !d->collectionRestriction )
-    {
-        d->includedBuilder = false;
-        d->collectionRestriction = true;
-    }
-    if( m_collection->collectionId() == collectionId )
-        d->includedBuilder = true;
-    return this;
-}
-
-QueryMaker*
-SqlQueryMaker::excludeCollection( const QString &collectionId )
-{
-    d->collectionRestriction = true;
-    if( m_collection->collectionId() == collectionId )
-        d->includedBuilder = false;
-    return this;
-}
-
 QueryMaker*
 SqlQueryMaker::addMatch( const Meta::TrackPtr &track )
 {
@@ -391,10 +344,13 @@ SqlQueryMaker::addMatch( const Meta::TrackPtr &track )
     return this;
 }
 
+
 QueryMaker*
-SqlQueryMaker::addMatch( const Meta::ArtistPtr &artist )
+SqlQueryMaker::addMatch( const Meta::ArtistPtr &artist, ArtistMatchBehaviour behaviour )
 {
     d->linkedTables |= Private::ARTIST_TAB;
+    if( behaviour == AlbumArtists || behaviour == AlbumOrTrackArtists )
+        d->linkedTables |= Private::ALBUMARTIST_TAB;
 
     QString artistQuery;
     QString albumArtistQuery;
@@ -410,7 +366,7 @@ SqlQueryMaker::addMatch( const Meta::ArtistPtr &artist )
         albumArtistQuery = "( albumartists.name IS NULL OR albumartists.name = '')";
     }
 
-    switch( d->artistMode )
+    switch( behaviour )
     {
     case TrackArtists:
         d->queryMatch += " AND " + artistQuery;
@@ -422,8 +378,6 @@ SqlQueryMaker::addMatch( const Meta::ArtistPtr &artist )
         d->queryMatch += " AND ( (" + artistQuery + " ) OR ( " + albumArtistQuery + " ) )";
         break;
     }
-    //Turn back default value, so we don't need to worry about this until the next AlbumArtist request.
-    d->artistMode = TrackArtists;
     return this;
 }
 
@@ -640,7 +594,7 @@ SqlQueryMaker::addReturnFunction( ReturnFunction function, qint64 value )
                 sqlfunction = "MIN";
                 break;
             default:
-                sqlfunction = "Unknown function in SqlQueryMaker::addReturnFunction, function was: " + function;
+                sqlfunction = "Unknown function in SqlQueryMaker::addReturnFunction, function was: " + QString::number( function );
         }
         d->queryReturnValues += QString( "%1(%2)" ).arg( sqlfunction, nameForValue( value ) );
         d->returnValueType = value;
@@ -660,13 +614,6 @@ SqlQueryMaker::orderBy( qint64 value, bool descending )
     return this;
 }
 
-QueryMaker*
-SqlQueryMaker::orderByRandom()
-{
-    d->queryOrderBy = " ORDER BY " + m_collection->sqlStorage()->randomFunc();
-    return this;
-}
-
 QueryMaker*
 SqlQueryMaker::limitMaxResultSize( int size )
 {
@@ -685,16 +632,6 @@ SqlQueryMaker::setAlbumQueryMode( AlbumQueryMode mode )
     return this;
 }
 
-QueryMaker*
-SqlQueryMaker::setArtistQueryMode( ArtistQueryMode mode )
-{
-    if( mode == AlbumArtists || mode == AlbumOrTrackArtists )
-        d->linkedTables |= Private::ALBUMARTIST_TAB;
-
-    d->artistMode = mode;
-    return this;
-}
-
 QueryMaker*
 SqlQueryMaker::setLabelQueryMode( LabelQueryMode mode )
 {
@@ -817,6 +754,15 @@ SqlQueryMaker::linkTables()
                         d->linkedTables ^= Private::URLS_TAB;
                     break;
                 }
+                case Meta::valArtist:
+               {
+                    d->queryFrom += " artists";
+                    if( d->linkedTables & Private::ARTIST_TAB )
+                        d->linkedTables ^= Private::ARTIST_TAB;
+                    if( d->linkedTables & Private::URLS_TAB )
+                        d->linkedTables ^= Private::URLS_TAB;
+                    break;
+                }
                 case Meta::valGenre:
                 {
                     d->queryFrom += " genres";
@@ -877,7 +823,7 @@ SqlQueryMaker::buildQuery()
     query += " FROM ";
     query += d->queryFrom;
 
-    //dynamic collection
+    // dynamic collection (only mounted file systems are considered)
     if( (d->linkedTables & Private::URLS_TAB) && m_collection->mountPointManager() )
     {
         query += " WHERE 1 ";
@@ -968,59 +914,45 @@ SqlQueryMaker::collectionIds() const
     return list;
 }
 
-Meta::DataList
-SqlQueryMaker::data( const QString &id ) const
-{
-    Q_UNUSED( id );
-    return d->blockingData;
-}
-
 Meta::TrackList
-SqlQueryMaker::tracks( const QString &id ) const
+SqlQueryMaker::tracks() const
 {
-    Q_UNUSED( id );
     return d->blockingTracks;
 }
 
 Meta::AlbumList
-SqlQueryMaker::albums( const QString &id ) const
+SqlQueryMaker::albums() const
 {
-    Q_UNUSED( id );
     return d->blockingAlbums;
 }
 
 Meta::ArtistList
-SqlQueryMaker::artists( const QString &id ) const
+SqlQueryMaker::artists() const
 {
-    Q_UNUSED( id );
     return d->blockingArtists;
 }
 
 Meta::GenreList
-SqlQueryMaker::genres( const QString &id ) const
+SqlQueryMaker::genres() const
 {
-    Q_UNUSED( id );
     return d->blockingGenres;
 }
 
 Meta::ComposerList
-SqlQueryMaker::composers( const QString &id ) const
+SqlQueryMaker::composers() const
 {
-    Q_UNUSED( id );
     return d->blockingComposers;
 }
 
 Meta::YearList
-SqlQueryMaker::years( const QString &id ) const
+SqlQueryMaker::years() const
 {
-    Q_UNUSED( id );
     return d->blockingYears;
 }
 
 QStringList
-SqlQueryMaker::customData( const QString &id ) const
+SqlQueryMaker::customData() const
 {
-    Q_UNUSED( id );
     return d->blockingCustomData;
 }
 
@@ -1114,7 +1046,7 @@ SqlQueryMaker::nameForValue( qint64 value )
             d->linkedTables |= Private::TAGS_TAB;
             return "tracks.modifydate";
         default:
-            return "ERROR: unknown value in SqlQueryMaker::nameForValue(qint64): value=" + value;
+            return "ERROR: unknown value in SqlQueryMaker::nameForValue(qint64): value=" + QString::number( value );
     }
 }
 
@@ -1169,70 +1101,55 @@ SqlQueryMaker::likeCondition( const QString &text, bool anyBegin, bool anyEnd )
     }
     else
     {
-        return QString( " = '%1' " ).arg( escape( text ) );
+        return QString( " = '%1' COLLATE utf8_unicode_ci " ).arg( escape( text ) );
     }
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::AlbumList &albums)
+SqlQueryMaker::blockingNewResultReady(const Meta::AlbumList &albums)
 {
-    Q_UNUSED( id );
     d->blockingAlbums = albums;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::ArtistList &artists)
+SqlQueryMaker::blockingNewResultReady(const Meta::ArtistList &artists)
 {
-    Q_UNUSED( id );
     d->blockingArtists = artists;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::GenreList &genres)
+SqlQueryMaker::blockingNewResultReady(const Meta::GenreList &genres)
 {
-    Q_UNUSED( id );
     d->blockingGenres = genres;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::ComposerList &composers)
+SqlQueryMaker::blockingNewResultReady(const Meta::ComposerList &composers)
 {
-    Q_UNUSED( id );
     d->blockingComposers = composers;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::YearList &years)
+SqlQueryMaker::blockingNewResultReady(const Meta::YearList &years)
 {
-    Q_UNUSED( id );
     d->blockingYears = years;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::TrackList &tracks)
+SqlQueryMaker::blockingNewResultReady(const Meta::TrackList &tracks)
 {
-    Q_UNUSED( id );
     d->blockingTracks = tracks;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const Meta::DataList &data)
+SqlQueryMaker::blockingNewResultReady(const QStringList &customData)
 {
-    Q_UNUSED( id );
-    d->blockingData = data;
-}
-
-void
-SqlQueryMaker::blockingNewResultReady(const QString &id, const QStringList &customData)
-{
-    Q_UNUSED( id );
     d->blockingCustomData = customData;
 }
 
 void
-SqlQueryMaker::blockingNewResultReady( const QString &id, const Meta::LabelList &labels )
+SqlQueryMaker::blockingNewResultReady(const Meta::LabelList &labels )
 {
-    Q_UNUSED( id );
     d->blockingLabels = labels;
 }
 
