@@ -81,7 +81,7 @@ TrackList TrackMatcher::match( const TrackList &tracks )
 
 
 
-ArtistMatcher::ArtistMatcher( ArtistPtr artist, Collections::QueryMaker::ArtistQueryMode artistMode )
+ArtistMatcher::ArtistMatcher( ArtistPtr artist, Collections::QueryMaker::ArtistMatchBehaviour artistMode )
     : MemoryMatcher()
     , m_artist( artist )
     , m_queryMode( artistMode )
@@ -102,7 +102,7 @@ TrackList ArtistMatcher::match( Collections::MemoryCollection *memColl )
     {
         case Collections::QueryMaker::AlbumOrTrackArtists:
         case Collections::QueryMaker::AlbumArtists:
-            foreach( AlbumPtr album, memColl->albumMap().values() )
+            foreach( AlbumPtr album, memColl->albumMap() )
                 if( album->albumArtist() == artist )
                     matchingTracks.append( album->tracks() );
 
