@@ -19,7 +19,7 @@
 #include "File.h"
 #include "File_p.h"
 
-#include "config.h"
+#include <config.h>
 #ifdef HAVE_LIBLASTFM
 #include "LastfmReadLabelCapability.h"
 #endif
@@ -592,3 +592,5 @@ Track::commitIfInNonBatchUpdate()
     notifyObservers();
     d->lock.lockForWrite(); // return to original state
 }
+
+#include "File_p.moc"
