@@ -18,17 +18,17 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "CoverFoundDialog"
+
 #include "CoverFoundDialog.h"
 
-#define DEBUG_PREFIX "CoverFoundDialog"
-#include "core/support/Debug.h"
-
-#include "AlbumBreadcrumbWidget.h"
-#include "core/support/Amarok.h"
-#include "CoverViewDialog.h"
-#include "PixmapViewer.h"
-#include "statusbar/KJobProgressBar.h"
 #include "SvgHandler.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "covermanager/CoverViewDialog.h"
+#include "statusbar/KJobProgressBar.h"
+#include "widgets/AlbumBreadcrumbWidget.h"
+#include "widgets/PixmapViewer.h"
 
 #include <KComboBox>
 #include <KConfigGroup>
@@ -86,8 +86,8 @@ CoverFoundDialog::CoverFoundDialog( const CoverFetchUnit::Ptr unit,
     breadcrumbLabel->setFont( breadcrumbLabelFont );
     breadcrumbLabel->setIndent( 4 );
 
-    connect( breadcrumb, SIGNAL(artistClicked(const QString&)), SLOT(addToCustomSearch(const QString&)) );
-    connect( breadcrumb, SIGNAL(albumClicked(const QString&)), SLOT(addToCustomSearch(const QString&)) );
+    connect( breadcrumb, SIGNAL(artistClicked(QString)), SLOT(addToCustomSearch(QString)) );
+    connect( breadcrumb, SIGNAL(albumClicked(QString)), SLOT(addToCustomSearch(QString)) );
 
     KHBox *searchBox = new KHBox( vbox );
     vbox->setSpacing( 4 );
@@ -127,15 +127,12 @@ CoverFoundDialog::CoverFoundDialog( const CoverFetchUnit::Ptr unit,
     QMenu *sourceMenu = new QMenu( sourceButton );
     QAction *lastFmAct = new QAction( i18n( "Last.fm" ), sourceMenu );
     QAction *googleAct = new QAction( i18n( "Google" ), sourceMenu );
-    QAction *yahooAct = new QAction( i18n( "Yahoo!" ), sourceMenu );
     QAction *discogsAct = new QAction( i18n( "Discogs" ), sourceMenu );
     lastFmAct->setCheckable( true );
     googleAct->setCheckable( true );
-    yahooAct->setCheckable( true );
     discogsAct->setCheckable( true );
     connect( lastFmAct, SIGNAL(triggered()), this, SLOT(selectLastFm()) );
     connect( googleAct, SIGNAL(triggered()), this, SLOT(selectGoogle()) );
-    connect( yahooAct, SIGNAL(triggered()), this, SLOT(selectYahoo()) );
     connect( discogsAct, SIGNAL(triggered()), this, SLOT(selectDiscogs()) );
 
     m_sortAction = new QAction( i18n( "Sort by size" ), sourceMenu );
@@ -145,17 +142,16 @@ CoverFoundDialog::CoverFoundDialog( const CoverFetchUnit::Ptr unit,
     QActionGroup *ag = new QActionGroup( sourceButton );
     ag->addAction( lastFmAct );
     ag->addAction( googleAct );
-    ag->addAction( yahooAct );
     ag->addAction( discogsAct );
     sourceMenu->addActions( ag->actions() );
     sourceMenu->addSeparator();
     sourceMenu->addAction( m_sortAction );
     sourceButton->setMenu( sourceMenu );
 
-    connect( m_search, SIGNAL(returnPressed(const QString&)), SLOT(insertComboText(const QString&)) );
-    connect( m_search, SIGNAL(returnPressed(const QString&)), SLOT(processQuery(const QString&)) );
-    connect( m_search, SIGNAL(returnPressed(const QString&)), SLOT(updateSearchButton(const QString&)) );
-    connect( m_search, SIGNAL(editTextChanged(const QString&)), SLOT(updateSearchButton(const QString&)) );
+    connect( m_search, SIGNAL(returnPressed(QString)), SLOT(insertComboText(QString)) );
+    connect( m_search, SIGNAL(returnPressed(QString)), SLOT(processQuery(QString)) );
+    connect( m_search, SIGNAL(returnPressed(QString)), SLOT(updateSearchButton(QString)) );
+    connect( m_search, SIGNAL(editTextChanged(QString)), SLOT(updateSearchButton(QString)) );
     connect( m_search->lineEdit(), SIGNAL(clearButtonClicked()), SLOT(clearQueryButtonClicked()));
     connect( m_searchButton, SIGNAL(clicked()), SLOT(processQuery()) );
 
@@ -172,12 +168,12 @@ CoverFoundDialog::CoverFoundDialog( const CoverFetchUnit::Ptr unit,
     m_view->setViewMode( QListView::IconMode );
     m_view->setResizeMode( QListView::Adjust );
 
-    connect( m_view, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)),
-             this,   SLOT(currentItemChanged(QListWidgetItem*, QListWidgetItem*)) );
+    connect( m_view, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
+             this,   SLOT(currentItemChanged(QListWidgetItem*,QListWidgetItem*)) );
     connect( m_view, SIGNAL(itemDoubleClicked(QListWidgetItem*)),
              this,   SLOT(itemDoubleClicked(QListWidgetItem*)) );
-    connect( m_view, SIGNAL(customContextMenuRequested(const QPoint&)),
-             this,   SLOT(itemMenuRequested(const QPoint&)) );
+    connect( m_view, SIGNAL(customContextMenuRequested(QPoint)),
+             this,   SLOT(itemMenuRequested(QPoint)) );
 
     splitter->addWidget( m_sideBar );
     splitter->addWidget( vbox );
@@ -192,8 +188,6 @@ CoverFoundDialog::CoverFoundDialog( const CoverFetchUnit::Ptr unit,
 
     if( source == "LastFm" )
         lastFmAct->setChecked( true );
-    else if( source == "Yahoo" )
-        yahooAct->setChecked( true );
     else if( source == "Discogs" )
         discogsAct->setChecked( true );
     else
@@ -202,13 +196,16 @@ CoverFoundDialog::CoverFoundDialog( const CoverFetchUnit::Ptr unit,
     typedef CoverFetchArtPayload CFAP;
     const CFAP *payload = dynamic_cast< const CFAP* >( unit->payload() );
     if( !m_album->hasImage() )
-        m_sideBar->setPixmap( m_album->image(190) );
+        m_sideBar->setPixmap( QPixmap::fromImage( m_album->image(190 ) ) );
     else if( payload )
         add( m_album->image(), data, payload->imageSize() );
     else
         add( m_album->image(), data );
     m_view->setCurrentItem( m_view->item( 0 ) );
     updateGui();
+    
+    connect( The::networkAccessManager(), SIGNAL(requestRedirected(QNetworkReply*,QNetworkReply*)),
+             this, SLOT(fetchRequestRedirected(QNetworkReply*,QNetworkReply*)) );
 }
 
 CoverFoundDialog::~CoverFoundDialog()
@@ -227,7 +224,7 @@ void CoverFoundDialog::hideEvent( QHideEvent *event )
     event->accept();
 }
 
-void CoverFoundDialog::add( const QPixmap &cover,
+void CoverFoundDialog::add( const QImage &cover,
                             const CoverFetch::Metadata &metadata,
                             const CoverFetch::ImageSize imageSize )
 {
@@ -329,9 +326,9 @@ void CoverFoundDialog::currentItemChanged( QListWidgetItem *current, QListWidget
     if( !current )
         return;
     CoverFoundItem *it = static_cast< CoverFoundItem* >( current );
-    QPixmap pixmap = it->hasBigPix() ? it->bigPix() : it->thumb();
-    m_image = pixmap;
-    m_sideBar->setPixmap( pixmap, it->metadata() );
+    QImage image = it->hasBigPix() ? it->bigPix() : it->thumb();
+    m_image = image;
+    m_sideBar->setPixmap( QPixmap::fromImage(image), it->metadata() );
 }
 
 void CoverFoundDialog::itemDoubleClicked( QListWidgetItem *item )
@@ -369,7 +366,14 @@ void CoverFoundDialog::saveAs()
     if( !item->hasBigPix() && !fetchBigPix() )
         return;
 
-    KFileDialog dlg( m_album->tracks().first()->playableUrl().directory(), QString(), this );
+    Meta::TrackList tracks = m_album->tracks();
+    if( tracks.isEmpty() )
+    {
+        warning() << "no tracks associated with album" << m_album->name();
+        return;
+    }
+
+    KFileDialog dlg( tracks.first()->playableUrl().directory(), QString(), this );
     dlg.setCaption( i18n("Cover Image Save Location") );
     dlg.setMode( KFile::File | KFile::LocalOnly );
     dlg.setOperationMode( KFileDialog::Saving );
@@ -401,14 +405,14 @@ void CoverFoundDialog::saveAs()
         return;
     }
 
-    const QPixmap &pixmap = item->bigPix();
+    const QImage &image = item->bigPix();
     const QString &ext = KMimeType::extractKnownExtension( saveUrl.path() ).toLower();
     if( ext == "jpg" || ext == "jpeg" )
-        pixmap.save( &saveFile, "JPG" );
+        image.save( &saveFile, "JPG" );
     else if( ext == "png" )
-        pixmap.save( &saveFile, "PNG" );
+        image.save( &saveFile, "PNG" );
     else
-        pixmap.save( &saveFile );
+        image.save( &saveFile );
 
     if( (saveFile.size() == 0) || !saveFile.finalize() )
     {
@@ -446,23 +450,45 @@ void CoverFoundDialog::slotButtonClicked( int button )
     }
 }
 
+void CoverFoundDialog::fetchRequestRedirected( QNetworkReply *oldReply,
+                                               QNetworkReply *newReply )
+{
+    KUrl oldUrl = oldReply->request().url();
+    KUrl newUrl = newReply->request().url();
+
+    // Since we were redirected we have to check if the redirect
+    // was for one of our URLs and if the new URL is not handled
+    // already.
+    if( m_urls.contains( oldUrl ) && !m_urls.contains( newUrl ) )
+    {
+        // Get the unit for the old URL.
+        CoverFoundItem *item = m_urls.value( oldUrl );
+
+        // Add the unit with the new URL and remove the old one.
+        m_urls.insert( newUrl, item );
+        m_urls.remove( oldUrl );
+    }
+}
+
 void CoverFoundDialog::handleFetchResult( const KUrl &url, QByteArray data,
                                           NetworkAccessManagerProxy::Error e )
 {
     CoverFoundItem *item = m_urls.take( url );
-    QPixmap pixmap;
-    if( e.code == QNetworkReply::NoError && pixmap.loadFromData( data ) )
+    QImage image;
+    if( item && e.code == QNetworkReply::NoError && image.loadFromData( data ) )
     {
-        item->setBigPix( pixmap );
-        m_sideBar->setPixmap( pixmap );
-        m_dialog.data()->accept();
+        item->setBigPix( image );
+        m_sideBar->setPixmap( QPixmap::fromImage( image ) );
+        if( m_dialog )
+            m_dialog.data()->accept();
     }
     else
     {
         QStringList errors;
         errors << e.description;
         KMessageBox::errorList( this, i18n("Sorry, the cover image could not be retrieved."), errors );
-        m_dialog.data()->reject();
+        if( m_dialog )
+            m_dialog.data()->reject();
     }
 }
 
@@ -517,8 +543,8 @@ void CoverFoundDialog::display()
     if( !success )
         return;
 
-    const QPixmap &pixmap = item->hasBigPix() ? item->bigPix() : item->thumb();
-    QWeakPointer<CoverViewDialog> dlg = new CoverViewDialog( pixmap, this );
+    const QImage &image = item->hasBigPix() ? item->bigPix() : item->thumb();
+    QWeakPointer<CoverViewDialog> dlg = new CoverViewDialog( image, this );
     dlg.data()->show();
     dlg.data()->raise();
     dlg.data()->activateWindow();
@@ -534,22 +560,16 @@ void CoverFoundDialog::processQuery( const QString &input )
 {
     const bool inputEmpty( input.isEmpty() );
     const bool mQueryEmpty( m_query.isEmpty() );
-    bool incrementPage( false );
 
     QString q;
     if( inputEmpty && !mQueryEmpty )
     {
         q = m_query;
-        incrementPage = true;
     }
     else if( !inputEmpty || !mQueryEmpty )
     {
         q = input;
-        if( m_query == input )
-        {
-            incrementPage = true;
-        }
-        else
+        if( m_query != input )
         {
             m_query = input;
             m_queryPage = 0;
@@ -584,16 +604,6 @@ void CoverFoundDialog::selectLastFm()
     debug() << "Select Last.fm as source";
 }
 
-void CoverFoundDialog::selectYahoo()
-{
-    KConfigGroup config = Amarok::config( "Cover Fetcher" );
-    config.writeEntry( "Interactive Image Source", "Yahoo" );
-    m_sortAction->setEnabled( true );
-    m_queryPage = 0;
-    processQuery();
-    debug() << "Select Yahoo! as source";
-}
-
 void CoverFoundDialog::selectGoogle()
 {
     KConfigGroup config = Amarok::config( "Cover Fetcher" );
@@ -705,7 +715,7 @@ CoverFoundSideBar::CoverFoundSideBar( const Meta::AlbumPtr album, QWidget *paren
     m_tabs->addTab( metaArea, i18n( "Information" ) );
     m_tabs->addTab( m_notes, i18n( "Notes" ) );
     setMaximumWidth( 200 );
-    setPixmap( m_album->image( 190 ) );
+    setPixmap( QPixmap::fromImage( m_album->image( 190 ) ) );
     clear();
 }
 
@@ -800,7 +810,6 @@ void CoverFoundSideBar::updateMetaTable()
     }
 
     QString refUrl;
-    QString refShowUrl; // only used by Yahoo atm
 
     const QString source = m_metadata.value( "source" );
     if( source == "Last.fm" || source == "Discogs" )
@@ -811,17 +820,12 @@ void CoverFoundSideBar::updateMetaTable()
     {
         refUrl = m_metadata.value( "imgrefurl" );
     }
-    else if( source == "Yahoo!" )
-    {
-        refUrl = m_metadata.value( "refererclickurl" );
-        refShowUrl = m_metadata.value( "refererurl" );
-    }
 
     if( !refUrl.isEmpty() )
     {
         QFont font;
         QFontMetrics qfm( font );
-        const QString &toolUrl = refShowUrl.isEmpty() ? refUrl : refShowUrl;
+        const QString &toolUrl = refUrl;
         const QString &tooltip = qfm.elidedText( toolUrl, Qt::ElideMiddle, 350 );
         const QString &decoded = QUrl::fromPercentEncoding( refUrl.toLocal8Bit() );
         const QString &url     = QString( "<a href=\"%1\">%2</a>" )
@@ -849,7 +853,7 @@ void CoverFoundSideBar::clearMetaTable()
     }
 }
 
-CoverFoundItem::CoverFoundItem( const QPixmap &cover,
+CoverFoundItem::CoverFoundItem( const QImage &cover,
                                 const CoverFetch::Metadata &data,
                                 const CoverFetch::ImageSize imageSize,
                                 QListWidget *parent )
@@ -867,7 +871,7 @@ CoverFoundItem::CoverFoundItem( const QPixmap &cover,
         break;
     }
 
-    QPixmap scaledPix = cover.scaled( QSize( 120, 120 ), Qt::KeepAspectRatio );
+    QPixmap scaledPix = QPixmap::fromImage(cover.scaled( QSize( 120, 120 ), Qt::KeepAspectRatio ));
     QPixmap prettyPix = The::svgHandler()->addBordersToPixmap( scaledPix, 5, QString(), true );
     setSizeHint( QSize( 140, 150 ) );
     setIcon( prettyPix );
@@ -902,8 +906,6 @@ void CoverFoundItem::setCaption()
     if( size )
     {
         const QString source = m_metadata.value( QLatin1String("source") );
-        if( source == QLatin1String("Yahoo!") )
-            size /= 1024;
 
         captions << ( QString::number( size ) + QLatin1Char('k') );
     }
