@@ -16,10 +16,12 @@
 
 #include "DatabaseImporterDialog.h"
 
-#include "Debug.h"
-#include "databaseimporter/amarok14/FastForwardImporter.h"
-#include "databaseimporter/itunes/ITunesImporter.h"
+#include "core/meta/Meta.h"
+#include "core/support/Debug.h"
+#include "databaseimporter/SqlBatchImporter.h"
+#include "databaseimporter/SqlBatchImporterConfig.h"
 
+#include <KLocale>
 #include <KPageWidgetItem>
 #include <KVBox>
 
@@ -39,56 +41,34 @@ DatabaseImporterDialog::DatabaseImporterDialog( QWidget *parent )
     KVBox *importerBox = new KVBox( this );
     importerBox->setSpacing( KDialog::spacingHint() );
 
-    QString text = i18n("This tool allows you to import track information and<br>statistical data from another music application.");
-    text += i18n("<br><br>Any statistical data in your database will be <i>overwritten</i>" );
-    QLabel *label = new QLabel( text, importerBox );
-    label->setTextFormat( Qt::RichText );
-    label->setAlignment( Qt::AlignHCenter );
-    label->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Minimum ); // Don't stretch vertically
-
-    m_buttons = new QButtonGroup( importerBox );
-    m_buttons->setExclusive( true );
-
-    QRadioButton *amarok = new QRadioButton( i18n("Amarok 1.4"), importerBox );
-    QRadioButton *itunes = new QRadioButton( i18n("iTunes"), importerBox );
-    QRadioButton *banshee = new QRadioButton( i18n("Banshee"), importerBox );
-    QRadioButton *rhythmbox = new QRadioButton( i18n("Rhythmbox"), importerBox );
-
-    amarok->setChecked( true );
-    itunes->setEnabled( true );
-    banshee->setEnabled( false );
-    rhythmbox->setEnabled( false );
-
-    banshee->setHidden( true );
-    rhythmbox->setHidden( true );
-
-    m_buttons->addButton( amarok );
-    m_buttons->addButton( itunes );
-    m_buttons->addButton( banshee );
-    m_buttons->addButton( rhythmbox );
-
-    m_buttonHash.insert( amarok, FastForwardImporter::name() );
-    m_buttonHash.insert( itunes, ITunesImporter::name() );
-    m_buttonHash.insert( banshee, "" );
-    m_buttonHash.insert( rhythmbox, "" );
-    
-    m_selectImporterPage = addPage( importerBox, i18n("Select Importer") );
-
     m_configBox = new KVBox( this );
     m_configBox->setSpacing( KDialog::spacingHint() );
 
-    m_configPage = addPage( m_configBox, i18n("Configuration") );
+    m_configPage = addPage( m_configBox, i18n("Import configuration") );
+
+    m_importer = new SqlBatchImporter( this );
+    connect( m_importer, SIGNAL(importSucceeded()), this, SLOT(importSucceeded()) );
+    connect( m_importer, SIGNAL(importFailed()), this, SLOT(importFailed()) );
+    connect( m_importer, SIGNAL(trackAdded(Meta::TrackPtr)), this, SLOT(importedTrack(Meta::TrackPtr)) );
+    connect( m_importer, SIGNAL(trackDiscarded(QString)), this, SLOT(discardedTrack(QString)) );
+    connect( m_importer, SIGNAL(trackMatchFound(Meta::TrackPtr,QString)),
+             this, SLOT(matchedTrack(Meta::TrackPtr,QString)) );
+    connect( m_importer, SIGNAL(trackMatchMultiple(Meta::TrackList,QString)),
+             this, SLOT(ambigousTrack(Meta::TrackList,QString)) );
+    connect( m_importer, SIGNAL(importError(QString)), this, SLOT(importError(QString)) );
+    connect( m_importer, SIGNAL(showMessage(QString)), this, SLOT(showMessage(QString)) );
+    m_importerConfig = m_importer->configWidget( m_configBox );
 
     KVBox *resultBox = new KVBox( this );
     resultBox->setSpacing( KDialog::spacingHint() );
-   
+
     m_results = new QPlainTextEdit( resultBox );
     m_results->setReadOnly( true );
     m_results->setTabChangesFocus( true );
 
     m_resultsPage = addPage( resultBox, i18n("Migrating") );
 
-    connect( this, SIGNAL( currentPageChanged(KPageWidgetItem*, KPageWidgetItem*) ), SLOT( pageChanged( KPageWidgetItem*, KPageWidgetItem* ) ) );
+    connect( this, SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)), SLOT(pageChanged(KPageWidgetItem*,KPageWidgetItem*)) );
 }
 
 DatabaseImporterDialog::~DatabaseImporterDialog()
@@ -101,12 +81,6 @@ DatabaseImporterDialog::pageChanged( KPageWidgetItem *current, KPageWidgetItem *
 {
     DEBUG_BLOCK
 
-    if( before == m_selectImporterPage && current == m_configPage )
-    {
-        selectImporter();
-        return;
-    }
-
     if( before == m_configPage && current == m_resultsPage )
     {
         if( m_importer && !m_importer->importing() )
@@ -117,36 +91,6 @@ DatabaseImporterDialog::pageChanged( KPageWidgetItem *current, KPageWidgetItem *
     }
 }
 
-void
-DatabaseImporterDialog::selectImporter()
-{
-    DEBUG_BLOCK
-
-    QRadioButton *button = dynamic_cast<QRadioButton*>(m_buttons->checkedButton());
-    if( !button )
-        return;
-
-    QString name = m_buttonHash.value( button, QString() );
-    if( name.isEmpty() )
-        return;
-
-    delete m_importer;
-    m_importer = DatabaseImporterFactory::createImporter( name, this );
-    connect( m_importer, SIGNAL( importSucceeded() ), this, SLOT( importSucceeded() ) );
-    connect( m_importer, SIGNAL( importFailed() ), this, SLOT( importFailed() ) );
-    connect( m_importer, SIGNAL( trackAdded( Meta::TrackPtr ) ), this, SLOT( importedTrack( Meta::TrackPtr ) ) );
-    connect( m_importer, SIGNAL( trackDiscarded( QString ) ), this, SLOT( discardedTrack( QString ) ) );
-    connect( m_importer, SIGNAL( trackMatchFound( Meta::TrackPtr, QString ) ),
-             this, SLOT( matchedTrack( Meta::TrackPtr, QString ) ) );
-    connect( m_importer, SIGNAL( trackMatchMultiple( Meta::TrackList, QString ) ),
-             this, SLOT( ambigousTrack( Meta::TrackList, QString ) ) );
-    connect( m_importer, SIGNAL( importError( QString ) ), this, SLOT( importError( QString ) ) );
-    connect( m_importer, SIGNAL( showMessage( QString ) ), this, SLOT( showMessage( QString ) ) );
-
-    delete m_importerConfig;
-    m_importerConfig = m_importer->configWidget( m_configBox );
-}
-
 void
 DatabaseImporterDialog::importSucceeded()
 {
@@ -155,7 +99,7 @@ DatabaseImporterDialog::importSucceeded()
     if( !m_importer->importedCount() )
         text = i18n( "<b><font color='red'>Failed:</font></b> No tracks were imported" );
     else
-        text = i18np( "<b><font color='green'>Success:</font></b> Imported %1 track", 
+        text = i18np( "<b><font color='green'>Success:</font></b> Imported %1 track",
                       "<b><font color='green'>Success:</font></b> Imported %1 tracks", m_importer->importedCount() );
 
     m_results->appendHtml( text );
@@ -191,12 +135,19 @@ DatabaseImporterDialog::importedTrack( Meta::TrackPtr track )
     if( !track ) return;
 
     QString text;
-    if( !track->album() || track->album()->name().isEmpty() )
-        text = i18nc( "Track has been imported, format: Artist - Track", 
-                      "Imported <b>%1 - %2</b>", track->artist()->name(), track->name() );
+    Meta::ArtistPtr artist = track->artist();
+    Meta::AlbumPtr album = track->album();
+
+    if( !artist || artist->name().isEmpty() )
+        text = i18nc( "Track has been imported, format: Track",
+                      "Imported <b>%1</b>", track->name() );
+    else if( !album || album->name().isEmpty() )
+        text = i18nc( "Track has been imported, format: Artist - Track",
+                      "Imported <b>%1 - %2</b>", artist->name(), track->name() );
     else
-        text = i18nc( "Track has been imported, format: Artist - Track (Album)", 
-                      "Imported <b>%1 - %2 (%3)</b>", track->artist()->name(), track->name(), track->album()->name() );
+        text = i18nc( "Track has been imported, format: Artist - Track (Album)",
+                      "Imported <b>%1 - %2 (%3)</b>", artist->name(), track->name(), album->name() );
+
     m_results->appendHtml( text );
 }
 
@@ -213,13 +164,19 @@ void DatabaseImporterDialog::matchedTrack( Meta::TrackPtr track, QString oldUrl
     if( !track ) return;
 
     QString text;
+    Meta::ArtistPtr artist = track->artist();
+    Meta::AlbumPtr album = track->album();
+
     //TODO: help text; also check wording with imported; unify?
-    if( !track->album() || track->album()->name().isEmpty() )
+    if( !artist || artist->name().isEmpty() )
+        text = i18nc( "Track has been imported by tags, format: Track, from Url, to Url",
+                      "Imported <b><font color='green'>%1</font></b><br/>&nbsp;&nbsp;from %2<br/>&nbsp;&nbsp;to %3", track->name(), oldUrl, track->prettyUrl() );
+    else if( !album || album->name().isEmpty() )
         text = i18nc( "Track has been imported by tags, format: Artist - Track, from Url, to Url",
-                      "Imported <b><font color='green'>%1 - %2</font></b><br/>&nbsp;&nbsp;from %3<br/>&nbsp;&nbsp;to %4", track->artist()->name(), track->name(), oldUrl, track->prettyUrl() );
+                      "Imported <b><font color='green'>%1 - %2</font></b><br/>&nbsp;&nbsp;from %3<br/>&nbsp;&nbsp;to %4", artist->name(), track->name(), oldUrl, track->prettyUrl() );
     else
         text = i18nc( "Track has been imported by tags, format: Artist - Track (Album), from Url, to Url",
-                      "Imported <b><font color='green'>%1 - %2 (%3)</font></b><br/>&nbsp;&nbsp;from %4<br/>&nbsp;&nbsp;to %5", track->artist()->name(), track->name(), track->album()->name(), oldUrl, track->prettyUrl() );
+                      "Imported <b><font color='green'>%1 - %2 (%3)</font></b><br/>&nbsp;&nbsp;from %4<br/>&nbsp;&nbsp;to %5", artist->name(), track->name(), album->name(), oldUrl, track->prettyUrl() );
 
     m_results->appendHtml( text );
 }
