@@ -25,34 +25,35 @@
 
 #include "TagDialog.h"
 
+#include "MainWindow.h"
+#include "SvgHandler.h"
+#include "core/collections/QueryMaker.h"
+#include "core/interfaces/Logger.h"
+#include "core/meta/Statistics.h"
+#include "core/meta/TrackEditor.h"
+#include "core/meta/support/MetaUtility.h"
 #include "core/support/Amarok.h"
 #include "core/support/Components.h"
-#include "core/interfaces/Logger.h"
-#include "core-impl/collections/support/CollectionManager.h"
-#include "CoverLabel.h"
-#include "covermanager/CoverFetchingActions.h"
 #include "core/support/Debug.h"
-#include "core/capabilities/EditCapability.h"
-#include "FilenameLayoutDialog.h"
-#include "MainWindow.h"
-#include "MusicBrainzTagger.h"
-#include "core/meta/support/MetaUtility.h"
-#include "core/collections/QueryMaker.h"
-#include "SvgHandler.h"
-#include "TagGuesser.h"
-#include "ui_TagDialogBase.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "covermanager/CoverFetchingActions.h"
+#include "dialogs/MusicBrainzTagger.h"
+#include "widgets/CoverLabel.h"
+#include "widgets/FilenameLayoutWidget.h"
+#include "ui_TagDialogBase.h" // needs to be after including CoverLabel, silly
+#include "TagGuesserDialog.h"
 
 #include <KLineEdit>
 #include <KMenu>
 #include <KRun>
 
-
 namespace Meta {
 namespace Field {
     const QString LABELS = "labels";
     const QString LYRICS = "lyrics";
     const QString TYPE = "type";
     const QString COLLECTION = "collection";
+    const QString NOTE = "note";
 }
 }
 
@@ -108,8 +109,8 @@ TagDialog::TagDialog( Collections::QueryMaker *qm )
     resize( minimumSizeHint() );
 
     qm->setQueryType( Collections::QueryMaker::Track );
-    connect( qm, SIGNAL( newResultReady( Meta::TrackList ) ), this, SLOT( resultReady( Meta::TrackList ) ), Qt::QueuedConnection );
-    connect( qm, SIGNAL( queryDone() ), this, SLOT( queryDone() ), Qt::QueuedConnection );
+    connect( qm, SIGNAL(newResultReady(Meta::TrackList)), this, SLOT(resultReady(Meta::TrackList)), Qt::QueuedConnection );
+    connect( qm, SIGNAL(queryDone()), this, SLOT(queryDone()), Qt::QueuedConnection );
     qm->run();
 }
 
@@ -170,7 +171,7 @@ TagDialog::queryDone()
         initUi();
         setCurrentTrack( 0 );
 
-        QTimer::singleShot( 0, this, SLOT( show() ) );
+        QTimer::singleShot( 0, this, SLOT(show()) );
     }
     else
     {
@@ -412,79 +413,68 @@ TagDialog::labelSelected() //SLOT
     ui->removeButton->setEnabled( ui->labelsList->selectionModel()->hasSelection() );
 }
 
-//creates a KDialog and executes the FilenameLayoutDialog. Grabs a filename scheme, extracts tags (via TagGuesser) from filename and fills the appropriate fields on TagDialog.
+//creates a KDialog and executes the FilenameLayoutWidget. Grabs a filename scheme, extracts tags (via TagGuesser) from filename and fills the appropriate fields on TagDialog.
 void
 TagDialog::guessFromFilename() //SLOT
 {
-    KDialog dialog;
-    dialog.setCaption( i18n( "Filename Layout Chooser" ) );
-    dialog.setButtons( KDialog::Ok | KDialog::Cancel );
-    FilenameLayoutDialog widget( &dialog );
-    widget.setFileName( m_currentTrack->playableUrl().path() );
-    dialog.setMainWidget( &widget );
+    TagGuesserDialog dialog( m_currentTrack->playableUrl().path(), this );
 
     if( dialog.exec() == KDialog::Accepted )
     {
-        widget.onAccept();
+        dialog.onAccept();
 
         int cur = 0;
 
-        TagGuesser guesser;
-        guesser.setFilename( widget.getParsableFileName() );
-        guesser.setSchema( widget.getParsableScheme() );
-        guesser.setCaseType( widget.getCaseOptions() );
-        guesser.setConvertUnderscores( widget.getUnderscoreOptions() );
-        guesser.setCutTrailingSpaces( widget.getWhitespaceOptions() );
+        QMap<qint64,QString> tags = dialog.guessedTags();
 
-        if( guesser.guess() )
+        if( !tags.isEmpty() )
         {
-            QMap<qint64,QString> Tags = guesser.tags();
 
-            if( Tags.contains( Meta::valTitle ) )
-                ui->kLineEdit_title->setText( Tags[Meta::valTitle] );
+            if( tags.contains( Meta::valTitle ) )
+                ui->kLineEdit_title->setText( tags[Meta::valTitle] );
 
-            if( Tags.contains( Meta::valArtist ) )
+            if( tags.contains( Meta::valArtist ) )
             {
                 cur = ui->kComboBox_artist->currentIndex();
-                ui->kComboBox_artist->setItemText( cur, Tags[Meta::valArtist] );
+                ui->kComboBox_artist->setItemText( cur, tags[Meta::valArtist] );
             }
 
-            if( Tags.contains( Meta::valAlbum ) )
+            if( tags.contains( Meta::valAlbum ) )
             {
                 cur = ui->kComboBox_album->currentIndex();
-                ui->kComboBox_album->setItemText( cur, Tags[Meta::valAlbum] );
+                ui->kComboBox_album->setItemText( cur, tags[Meta::valAlbum] );
             }
 
-            if( Tags.contains( Meta::valAlbumArtist ) )
+            if( tags.contains( Meta::valAlbumArtist ) )
             {
                 cur = ui->kComboBox_albumArtist->currentIndex();
-                ui->kComboBox_albumArtist->setItemText( cur, Tags[Meta::valAlbumArtist] );
+                ui->kComboBox_albumArtist->setItemText( cur, tags[Meta::valAlbumArtist] );
             }
 
-            if( Tags.contains( Meta::valTrackNr ) )
-                ui->qSpinBox_track->setValue( Tags[Meta::valTrackNr].toInt() );
+            if( tags.contains( Meta::valTrackNr ) )
+                ui->qSpinBox_track->setValue( tags[Meta::valTrackNr].toInt() );
 
-            if( Tags.contains( Meta::valComment ) )
-                ui->qPlainTextEdit_comment->setPlainText( Tags[Meta::valComment] );
+            if( tags.contains( Meta::valComment ) )
+                ui->qPlainTextEdit_comment->setPlainText( tags[Meta::valComment] );
 
-            if( Tags.contains( Meta::valYear ) )
-                ui->qSpinBox_year->setValue( Tags[Meta::valYear].toInt() );
+            if( tags.contains( Meta::valYear ) )
+                ui->qSpinBox_year->setValue( tags[Meta::valYear].toInt() );
 
-            if( Tags.contains( Meta::valComposer ) )
+            if( tags.contains( Meta::valComposer ) )
             {
                 cur = ui->kComboBox_composer->currentIndex();
-                ui->kComboBox_composer->setItemText( cur, Tags[Meta::valComposer] );
+                ui->kComboBox_composer->setItemText( cur, tags[Meta::valComposer] );
             }
 
-            if( Tags.contains( Meta::valGenre ) )
+            if( tags.contains( Meta::valGenre ) )
             {
                 cur = ui->kComboBox_genre->currentIndex();
-                ui->kComboBox_genre->setItemText( cur, Tags[Meta::valGenre] );
+                ui->kComboBox_genre->setItemText( cur, tags[Meta::valGenre] );
             }
 
-            if( Tags.contains( Meta::valDiscNr ) )
+            if( tags.contains( Meta::valDiscNr ) )
             {
-                ui->qSpinBox_discNumber->setValue( Tags[Meta::valDiscNr].toInt() );
+                ui->qSpinBox_discNumber->setValue( tags[Meta::valDiscNr].toInt() );
             }
         }
         else
@@ -545,49 +535,49 @@ void TagDialog::initUi()
     // set an icon for the open-in-konqui button
     ui->pushButton_open->setIcon( KIcon( "folder-amarok" ) );
 
-    connect( ui->pushButton_guessTags, SIGNAL(clicked()), SLOT( guessFromFilename() ) );
+    connect( ui->pushButton_guessTags, SIGNAL(clicked()), SLOT(guessFromFilename()) );
 
     // Connects for modification check
     // only set to overwrite-on-save if the text has changed
-    connect( ui->kLineEdit_title,       SIGNAL( textChanged( const QString& ) ),     SLOT( checkChanged() ) );
-    connect( ui->kComboBox_composer,    SIGNAL( activated( int ) ),                  SLOT( checkChanged() ) );
-    connect( ui->kComboBox_composer,    SIGNAL( editTextChanged( const QString& ) ), SLOT( checkChanged() ) );
-    connect( ui->kComboBox_artist,      SIGNAL( activated( int ) ),                  SLOT( checkChanged() ) );
-    connect( ui->kComboBox_artist,      SIGNAL( editTextChanged( const QString& ) ), SLOT( checkChanged() ) );
-    connect( ui->kComboBox_album,       SIGNAL( activated( int ) ),                  SLOT( checkChanged() ) );
-    connect( ui->kComboBox_album,       SIGNAL( editTextChanged( const QString& ) ), SLOT( checkChanged() ) );
-    connect( ui->kComboBox_albumArtist, SIGNAL( activated( int ) ),                  SLOT( checkChanged() ) );
-    connect( ui->kComboBox_albumArtist, SIGNAL( editTextChanged( const QString& ) ), SLOT( checkChanged() ) );
-    connect( ui->kComboBox_genre,       SIGNAL( activated( int ) ),                  SLOT( checkChanged() ) );
-    connect( ui->kComboBox_genre,       SIGNAL( editTextChanged( const QString& ) ), SLOT( checkChanged() ) );
-    connect( ui->kLineEdit_Bpm,         SIGNAL( textChanged( const QString& ) )    , SLOT( checkChanged() ) );
-    connect( ui->ratingWidget,          SIGNAL( ratingChanged( int ) ),              SLOT( checkChanged() ) );
-    connect( ui->qSpinBox_track,        SIGNAL( valueChanged( int ) ),               SLOT( checkChanged() ) );
-    connect( ui->qSpinBox_year,         SIGNAL( valueChanged( int ) ),               SLOT( checkChanged() ) );
-    connect( ui->qSpinBox_score,        SIGNAL( valueChanged( int ) ),               SLOT( checkChanged() ) );
-    connect( ui->qPlainTextEdit_comment,SIGNAL( textChanged() ),                     SLOT( checkChanged() ) );
-    connect( ui->kRichTextEdit_lyrics,  SIGNAL( textChanged() ),                     SLOT( checkChanged() ) );
-    connect( ui->qSpinBox_discNumber,   SIGNAL( valueChanged( int ) ),               SLOT( checkChanged() ) );
+    connect( ui->kLineEdit_title,       SIGNAL(textChanged(QString)),     SLOT(checkChanged()) );
+    connect( ui->kComboBox_composer,    SIGNAL(activated(int)),                  SLOT(checkChanged()) );
+    connect( ui->kComboBox_composer,    SIGNAL(editTextChanged(QString)), SLOT(checkChanged()) );
+    connect( ui->kComboBox_artist,      SIGNAL(activated(int)),                  SLOT(checkChanged()) );
+    connect( ui->kComboBox_artist,      SIGNAL(editTextChanged(QString)), SLOT(checkChanged()) );
+    connect( ui->kComboBox_album,       SIGNAL(activated(int)),                  SLOT(checkChanged()) );
+    connect( ui->kComboBox_album,       SIGNAL(editTextChanged(QString)), SLOT(checkChanged()) );
+    connect( ui->kComboBox_albumArtist, SIGNAL(activated(int)),                  SLOT(checkChanged()) );
+    connect( ui->kComboBox_albumArtist, SIGNAL(editTextChanged(QString)), SLOT(checkChanged()) );
+    connect( ui->kComboBox_genre,       SIGNAL(activated(int)),                  SLOT(checkChanged()) );
+    connect( ui->kComboBox_genre,       SIGNAL(editTextChanged(QString)), SLOT(checkChanged()) );
+    connect( ui->kLineEdit_Bpm,         SIGNAL(textChanged(QString))    , SLOT(checkChanged()) );
+    connect( ui->ratingWidget,          SIGNAL(ratingChanged(int)),              SLOT(checkChanged()) );
+    connect( ui->qSpinBox_track,        SIGNAL(valueChanged(int)),               SLOT(checkChanged()) );
+    connect( ui->qSpinBox_year,         SIGNAL(valueChanged(int)),               SLOT(checkChanged()) );
+    connect( ui->qSpinBox_score,        SIGNAL(valueChanged(int)),               SLOT(checkChanged()) );
+    connect( ui->qPlainTextEdit_comment,SIGNAL(textChanged()),                     SLOT(checkChanged()) );
+    connect( ui->kRichTextEdit_lyrics,  SIGNAL(textChanged()),                     SLOT(checkChanged()) );
+    connect( ui->qSpinBox_discNumber,   SIGNAL(valueChanged(int)),               SLOT(checkChanged()) );
 
-    connect( ui->pushButton_cancel,   SIGNAL( clicked() ), SLOT( cancelPressed() ) );
-    connect( ui->pushButton_ok,       SIGNAL( clicked() ), SLOT( accept() ) );
-    connect( ui->pushButton_open,     SIGNAL( clicked() ), SLOT( openPressed() ) );
-    connect( ui->pushButton_previous, SIGNAL( clicked() ), SLOT( previousTrack() ) );
-    connect( ui->pushButton_next,     SIGNAL( clicked() ), SLOT( nextTrack() ) );
-    connect( ui->checkBox_perTrack,   SIGNAL( toggled(bool) ), SLOT( perTrack(bool) ) );
+    connect( ui->pushButton_cancel,   SIGNAL(clicked()), SLOT(cancelPressed()) );
+    connect( ui->pushButton_ok,       SIGNAL(clicked()), SLOT(accept()) );
+    connect( ui->pushButton_open,     SIGNAL(clicked()), SLOT(openPressed()) );
+    connect( ui->pushButton_previous, SIGNAL(clicked()), SLOT(previousTrack()) );
+    connect( ui->pushButton_next,     SIGNAL(clicked()), SLOT(nextTrack()) );
+    connect( ui->checkBox_perTrack,   SIGNAL(toggled(bool)), SLOT(perTrack(bool)) );
 
-    connect( ui->addButton,           SIGNAL( clicked() ),                          SLOT( addLabelPressed() ) );
-    connect( ui->removeButton,        SIGNAL( clicked() ),                          SLOT( removeLabelPressed() ) );
-    connect( ui->kComboBox_label,     SIGNAL( activated( int ) ),                   SLOT( labelModified() ) );
-    connect( ui->kComboBox_label,     SIGNAL( editTextChanged( const QString& ) ),  SLOT( labelModified() ) );
-    connect( ui->kComboBox_label,     SIGNAL( returnPressed() ),                    SLOT( addLabelPressed() ) );
-    connect( ui->kComboBox_label,     SIGNAL( returnPressed() ),                    SLOT( checkChanged() ) );
-    connect( ui->labelsList,          SIGNAL( pressed( const QModelIndex& ) ),      SLOT( labelSelected() ) );
+    connect( ui->addButton,           SIGNAL(clicked()),                          SLOT(addLabelPressed()) );
+    connect( ui->removeButton,        SIGNAL(clicked()),                          SLOT(removeLabelPressed()) );
+    connect( ui->kComboBox_label,     SIGNAL(activated(int)),                   SLOT(labelModified()) );
+    connect( ui->kComboBox_label,     SIGNAL(editTextChanged(QString)),  SLOT(labelModified()) );
+    connect( ui->kComboBox_label,     SIGNAL(returnPressed()),                    SLOT(addLabelPressed()) );
+    connect( ui->kComboBox_label,     SIGNAL(returnPressed()),                    SLOT(checkChanged()) );
+    connect( ui->labelsList,          SIGNAL(pressed(QModelIndex)),      SLOT(labelSelected()) );
 
     ui->pixmap_cover->setContextMenuPolicy( Qt::CustomContextMenu );
-    connect( ui->pixmap_cover, SIGNAL( customContextMenuRequested(const QPoint &) ), SLOT( showCoverMenu(const QPoint &) ) );
+    connect( ui->pixmap_cover, SIGNAL(customContextMenuRequested(QPoint)), SLOT(showCoverMenu(QPoint)) );
 
-    connect( ui->pushButton_musicbrainz, SIGNAL( clicked() ), SLOT( musicbrainzTagger() ) );
+    connect( ui->pushButton_musicbrainz, SIGNAL(clicked()), SLOT(musicbrainzTagger()) );
 
     if( m_tracks.count() > 1 )
         setPerTrack( false );
@@ -632,7 +622,7 @@ TagDialog::startDataQuery( Collections::QueryMaker::QueryType type, const char *
     Collections::QueryMaker *qm = CollectionManager::instance()->queryMaker();
     qm->setQueryType( type );
 
-    connect( qm, SIGNAL( queryDone() ), SLOT( dataQueryDone() ), Qt::QueuedConnection );
+    connect( qm, SIGNAL(queryDone()), SLOT(dataQueryDone()), Qt::QueuedConnection );
     connect( qm, signal, slot,  Qt::QueuedConnection );
 
     qm->setAutoDelete( true );
@@ -643,20 +633,20 @@ void
 TagDialog::startDataQueries()
 {
     startDataQuery( Collections::QueryMaker::Artist,
-                    SIGNAL( newResultReady( Meta::ArtistList ) ),
-                    SLOT( resultReady( Meta::ArtistList ) ) );
+                    SIGNAL(newResultReady(Meta::ArtistList)),
+                    SLOT(resultReady(Meta::ArtistList)) );
     startDataQuery( Collections::QueryMaker::Album,
-                    SIGNAL( newResultReady( Meta::AlbumList ) ),
-                    SLOT( resultReady( Meta::AlbumList ) ) );
+                    SIGNAL(newResultReady(Meta::AlbumList)),
+                    SLOT(resultReady(Meta::AlbumList)) );
     startDataQuery( Collections::QueryMaker::Composer,
-                    SIGNAL( newResultReady( Meta::ComposerList ) ),
-                    SLOT( resultReady( Meta::ComposerList ) ) );
+                    SIGNAL(newResultReady(Meta::ComposerList)),
+                    SLOT(resultReady(Meta::ComposerList)) );
     startDataQuery( Collections::QueryMaker::Genre,
-                    SIGNAL( newResultReady( Meta::GenreList ) ),
-                    SLOT( resultReady( Meta::GenreList ) ) );
+                    SIGNAL(newResultReady(Meta::GenreList)),
+                    SLOT(resultReady(Meta::GenreList)) );
     startDataQuery( Collections::QueryMaker::Label,
-                    SIGNAL( newResultReady( Meta::LabelList ) ),
-                    SLOT( resultReady( Meta::LabelList ) ) );
+                    SIGNAL(newResultReady(Meta::LabelList)),
+                    SLOT(resultReady(Meta::LabelList)) );
 }
 
 
@@ -789,22 +779,32 @@ TagDialog::setTagsToUi( const QVariantMap &tags )
     ui->qSpinBox_score->setValue( tags.value( Meta::Field::SCORE ).toInt() );
     ui->ratingWidget->setRating( tags.value( Meta::Field::RATING ).toInt() );
     ui->ratingWidget->setMaxRating( 10 );
-    ui->qLabel_playcount->setText( unknownSafe( tags.value( Meta::Field::PLAYCOUNT ).toInt() ) );
+    int playcount = tags.value( Meta::Field::PLAYCOUNT ).toInt();
+    ui->qLabel_playcount->setText( unknownSafe( playcount ) );
 
-    QDate firstPlayed = tags.value( Meta::Field::FIRST_PLAYED ).toDate();
-    ui->qLabel_firstPlayed->setText( firstPlayed.isValid() ?
-                                     KGlobal::locale()->formatDate( firstPlayed, KLocale::ShortDate ) :
-                                     i18nc( "When this track first played", "Never") );
+    QDateTime firstPlayed = tags.value( Meta::Field::FIRST_PLAYED ).toDateTime();
+    ui->qLabel_firstPlayed->setText( Amarok::verboseTimeSince( firstPlayed ) );
 
-    QDate lastPlayed = tags.value( Meta::Field::LAST_PLAYED ).toDate();
-    ui->qLabel_lastPlayed->setText( lastPlayed.isValid() ?
-                                    KGlobal::locale()->formatDate( lastPlayed, KLocale::ShortDate ) :
-                                    i18nc( "When this track was last played", "Never") );
+    QDateTime lastPlayed = tags.value( Meta::Field::LAST_PLAYED ).toDateTime();
+    ui->qLabel_lastPlayed->setText( Amarok::verboseTimeSince( lastPlayed ) );
 
     ui->qLabel_collection->setText( tags.contains( Meta::Field::COLLECTION ) ?
                                     tags.value( Meta::Field::COLLECTION ).toString() :
                                     i18nc( "The collection this track is part of", "None") );
 
+    // special handling - we want to hide this if empty
+    if( tags.contains( Meta::Field::NOTE ) )
+    {
+        ui->noteLabel->show();
+        ui->qLabel_note->setText( tags.value( Meta::Field::NOTE ).toString() );
+        ui->qLabel_note->show();
+    }
+    else
+    {
+        ui->noteLabel->hide();
+        ui->qLabel_note->hide();
+    }
+
     ui->kRichTextEdit_lyrics->setTextOrHtml( tags.value( Meta::Field::LYRICS ).toString() );
 
     m_labelModel->setLabels( tags.value( Meta::Field::LABELS ).toStringList() );
@@ -829,7 +829,7 @@ TagDialog::setTagsToUi( const QVariantMap &tags )
     }
     else
     {
-        m_path = QString();
+        m_path.clear();
         ui->pushButton_open->setEnabled( false );
     }
 
@@ -942,11 +942,12 @@ TagDialog::getTagsFromTrack( const Meta::TrackPtr &track ) const
     if( track->filesize() )
         map.insert( Meta::Field::FILESIZE, track->filesize() );
 
-    map.insert( Meta::Field::SCORE, track->score() );
-    map.insert( Meta::Field::RATING, track->rating() );
-    map.insert( Meta::Field::PLAYCOUNT, track->playCount() );
-    map.insert( Meta::Field::FIRST_PLAYED, track->firstPlayed() );
-    map.insert( Meta::Field::LAST_PLAYED, track->lastPlayed() );
+    Meta::ConstStatisticsPtr statistics = track->statistics();
+    map.insert( Meta::Field::SCORE, statistics->score() );
+    map.insert( Meta::Field::RATING, statistics->rating() );
+    map.insert( Meta::Field::PLAYCOUNT, statistics->playCount() );
+    map.insert( Meta::Field::FIRST_PLAYED, statistics->firstPlayed() );
+    map.insert( Meta::Field::LAST_PLAYED, statistics->lastPlayed() );
     map.insert( Meta::Field::URL, track->prettyUrl() );
 
     map.insert( Meta::Field::TYPE, track->type() );
@@ -954,6 +955,10 @@ TagDialog::getTagsFromTrack( const Meta::TrackPtr &track ) const
     if( track->inCollection() )
         map.insert( Meta::Field::COLLECTION, track->collection()->prettyName() );
 
+    if( !track->notPlayableReason().isEmpty() )
+        map.insert( Meta::Field::NOTE, i18n( "The track is not playable. %1",
+                                             track->notPlayableReason() ) );
+
     QStringList labelNames;
     foreach( const Meta::LabelPtr &label, track->labels() )
     {
@@ -990,8 +995,8 @@ TagDialog::getTagsFromMultipleTracks() const
     if( map.value( Meta::Field::RATING ).toInt() )
         ratingCount++;
 
-    QDateTime firstPlayed = first->firstPlayed();
-    QDateTime lastPlayed = first->lastPlayed();
+    QDateTime firstPlayed = first->statistics()->firstPlayed();
+    QDateTime lastPlayed = first->statistics()->lastPlayed();
 
     qint64 length = first->length();
     qint64 size = first->filesize();
@@ -1009,7 +1014,7 @@ TagDialog::getTagsFromMultipleTracks() const
         mismatchingTags |= tags.keys().toSet() - map.keys().toSet();
 
         // - not the same in every file
-        foreach( QString key, (map.keys().toSet() & tags.keys().toSet()) )
+        foreach( const QString &key, (map.keys().toSet() & tags.keys().toSet()) )
         {
             if( map.value( key ) != tags.value( key ) )
                 mismatchingTags.insert( key );
@@ -1056,13 +1061,14 @@ TagDialog::getTagsFromMultipleTracks() const
         if( tags.value( Meta::Field::RATING ).toInt() )
             ratingCount++;
 
-        if( track->firstPlayed().isValid() &&
-            (!firstPlayed.isValid() || track->firstPlayed() < firstPlayed) )
-            firstPlayed = track->firstPlayed();
+        Meta::StatisticsPtr statistics = track->statistics();
+        if( statistics->firstPlayed().isValid() &&
+            (!firstPlayed.isValid() || statistics->firstPlayed() < firstPlayed) )
+            firstPlayed = statistics->firstPlayed();
 
-        if( track->lastPlayed().isValid() &&
-            (!lastPlayed.isValid() || track->lastPlayed() > lastPlayed) )
-            lastPlayed = track->lastPlayed();
+        if( statistics->lastPlayed().isValid() &&
+            (!lastPlayed.isValid() || statistics->lastPlayed() > lastPlayed) )
+            lastPlayed = statistics->lastPlayed();
 
         length += track->length();
         size += track->filesize();
@@ -1138,21 +1144,24 @@ TagDialog::setTagsToTrack()
             setTagsToMultipleTracks( newTags );
 
             // -- special handling for labels
-            // determine the differences
-            QSet<QString> oldLabelsSet = oldTags.value( Meta::Field::LABELS ).toStringList().toSet();
-            QSet<QString> newLabelsSet = newTags.value( Meta::Field::LABELS ).toStringList().toSet();
-
-            QSet<QString> labelsToRemove = oldLabelsSet - newLabelsSet;
-            QSet<QString> labelsToAdd = newLabelsSet - oldLabelsSet;
-
-            // apply the differences for each track
-            foreach( const Meta::TrackPtr &track, m_tracks )
+            if( newTags.contains( Meta::Field::LABELS ) )
             {
-                QSet<QString> labelsSet = m_storedTags[track].value( Meta::Field::LABELS ).toStringList().toSet();
-                labelsSet += labelsToAdd;
-                labelsSet -= labelsToRemove;
+                // determine the differences
+                QSet<QString> oldLabelsSet = oldTags.value( Meta::Field::LABELS ).toStringList().toSet();
+                QSet<QString> newLabelsSet = newTags.value( Meta::Field::LABELS ).toStringList().toSet();
 
-                m_storedTags[ track ].insert( Meta::Field::LABELS, QVariant( labelsSet.toList() ) );
+                QSet<QString> labelsToRemove = oldLabelsSet - newLabelsSet;
+                QSet<QString> labelsToAdd = newLabelsSet - oldLabelsSet;
+
+                // apply the differences for each track
+                foreach( const Meta::TrackPtr &track, m_tracks )
+                {
+                    QSet<QString> labelsSet = m_storedTags[track].value( Meta::Field::LABELS ).toStringList().toSet();
+                    labelsSet += labelsToAdd;
+                    labelsSet -= labelsToRemove;
+
+                    m_storedTags[ track ].insert( Meta::Field::LABELS, QVariant( labelsSet.toList() ) );
+                }
             }
         }
     }
@@ -1233,13 +1242,7 @@ TagDialog::updateCover()
 void
 TagDialog::setControlsAccessability()
 {
-    bool editable = true;
-    if( m_currentTrack )
-    {
-        QScopedPointer<Capabilities::EditCapability> ec(
-            m_currentTrack->create<Capabilities::EditCapability>() );
-        editable = ec && ec->isEditable();
-    }
+    bool editable = m_currentTrack ? bool( m_currentTrack->editor() ) : true;
 
     ui->kTabWidget->setTabEnabled( ui->kTabWidget->indexOf(ui->lyricsTab),
                                    m_perTrack );
@@ -1310,9 +1313,9 @@ TagDialog::saveTags()
             debug() << "File info changed....";
 
             if( data.contains( Meta::Field::SCORE ) )
-                track->setScore( data.value( Meta::Field::SCORE ).toInt() );
+                track->statistics()->setScore( data.value( Meta::Field::SCORE ).toInt() );
             if( data.contains( Meta::Field::RATING ) )
-                track->setRating( data.value( Meta::Field::RATING ).toInt() );
+                track->statistics()->setRating( data.value( Meta::Field::RATING ).toInt() );
             if( data.contains( Meta::Field::LYRICS ) )
             {
                 track->setCachedLyrics( data.value( Meta::Field::LYRICS ).toString() );
@@ -1321,20 +1324,14 @@ TagDialog::saveTags()
 
             saveLabels( track, data.value( Meta::Field::LABELS ).toStringList() );
 
-            QScopedPointer<Capabilities::EditCapability> ec( track->create<Capabilities::EditCapability>() );
+            Meta::TrackEditorPtr ec = track->editor();
             if( !ec )
             {
-                debug() << "Track does not have Capabilities::EditCapability. Aborting loop.";
-                continue;
-            }
-            if( !ec->isEditable() )
-            {
-                debug() << "Track not editable. Aborting loop.";
-                Amarok::Components::logger()->shortMessage( i18n( "Writing to file failed. Please check permissions and available disc space." ) );
+                debug() << "Track" << track->prettyUrl() << "does not have Meta::TrackEditor. Skiping.";
                 continue;
             }
 
-            ec->beginMetaDataUpdate();
+            ec->beginUpdate();
             if( data.contains( Meta::Field::TITLE ) )
                 ec->setTitle( data.value( Meta::Field::TITLE ).toString() );
             if( data.contains( Meta::Field::COMMENT ) )
@@ -1358,8 +1355,7 @@ TagDialog::saveTags()
             if( data.contains( Meta::Field::ALBUMARTIST ) )
                 ec->setAlbumArtist( data.value( Meta::Field::ALBUMARTIST ).toString() );
 
-            ec->endMetaDataUpdate();
-
+            ec->endUpdate();
             // note: the track should by itself emit a collectionUpdated signal if needed
         }
     }
@@ -1387,8 +1383,8 @@ TagDialog::musicbrainzTagger()
 
     MusicBrainzTagger *dialog = new MusicBrainzTagger( m_tracks, this );
     dialog->setWindowTitle( i18n( "MusicBrainz Tagger" ) );
-    connect( dialog, SIGNAL( sendResult( const QMap<Meta::TrackPtr,QVariantMap> ) ),
-             this, SLOT( musicbrainzTaggerResult( const QMap<Meta::TrackPtr,QVariantMap> ) ) );
+    connect( dialog, SIGNAL(sendResult(QMap<Meta::TrackPtr,QVariantMap>)),
+             this, SLOT(musicbrainzTaggerResult(QMap<Meta::TrackPtr,QVariantMap>)) );
     dialog->show();
 }
 
