@@ -74,14 +74,18 @@ fitsInOneLineHTML(const QString& text)
 }
 
 static QString
-breakLongLinesHTML(const QString& text)
+breakLongLinesHTML( const QString &origText )
 {
+    // filter-out HTML tags..
+    QString text( origText );
+    text.replace( '&', "&amp;" ); // needs to be first, obviously
+    text.replace( '<', "&lt;" );
+    text.replace( '>', "&gt;" );
+
     // Now let's break up long lines so that the tooltip doesn't become hideously large
-    if (fitsInOneLineHTML(text))
-    {
+    if( fitsInOneLineHTML( text ) )
         // If the text is not too long, return it as it is
         return text;
-    }
     else
     {
         const int lnSize = TOOLTIP_STATIC_LINEBREAK;
@@ -110,7 +114,7 @@ breakLongLinesHTML(const QString& text)
                     textInLines += wordPart + "<br/>";
                 }
             }
-            textInLines += word + " ";
+            textInLines += word + ' ';
             lineLength += word.size() + 1;
             words.removeFirst();
         }
@@ -491,10 +495,12 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case LastPlayed:
             {
-                if( statistics->lastPlayed().isValid() )
+                if( statistics->playCount() == 0 )
+                    return i18nc( "The amount of time since last played", "Never" );
+                else if( statistics->lastPlayed().isValid() )
                     return Amarok::verboseTimeSince( statistics->lastPlayed() );
                 else
-                    return i18nc( "The amount of time since last played", "Never" );
+                    return i18nc( "When this track was last played", "Unknown" );
             }
             case Length:
             {
