@@ -19,13 +19,11 @@
  ****************************************************************************************/
 
 #include "core-impl/collections/support/CollectionManager.h"
-#include "core/support/Debug.h"
 #include "core/collections/MetaQueryMaker.h"
 #include "core/collections/QueryMaker.h"
 #include "widgets/MetaQueryWidget.h"
 #include "widgets/kdatecombo.h"
-#include "widgets/kratingwidget.h"
-#include "shared/FileType.h"
+#include "FileType.h"
 
 #include <typeinfo>
 
@@ -40,12 +38,15 @@
 
 #include <KComboBox>
 #include <KIcon>
-#include <KDateTime> // for local time
+#include <KLocale>
+#include <klocalizeddate.h>
 #include <KNumInput>
-#include <klocale.h>
+#include <KRatingWidget>
 
 using namespace Amarok;
 
+static const int maxHours = 24;
+
 TimeDistanceWidget::TimeDistanceWidget( QWidget *parent )
     : QWidget( parent )
 {
@@ -55,7 +56,7 @@ TimeDistanceWidget::TimeDistanceWidget( QWidget *parent )
 
     m_unitSelection = new KComboBox(this);
     connect( m_timeEdit, SIGNAL(valueChanged(int)), this, SLOT(slotUpdateComboBoxLabels(int)) );
-    for (int i = 0; i < 5; ++i) {
+    for (int i = 0; i < 7; ++i) {
         m_unitSelection->addItem( QString() );
     }
     slotUpdateComboBoxLabels( 0 );
@@ -71,8 +72,15 @@ qint64 TimeDistanceWidget::timeDistance() const
     qint64 time = m_timeEdit->value();
     switch( m_unitSelection->currentIndex() )
     {
+    case 6:
+        time *= 365*24*60*60; // years
+        break;
+    case 5:
+        time *=  30*24*60*60; // months
+        break;
     case 4:
-        time *= 30; // months
+        time *=   7*24*60*60; // weeks
+        break;
     case 3:
         time *= 24; // days
     case 2:
@@ -89,20 +97,26 @@ void TimeDistanceWidget::setTimeDistance( qint64 value )
     // as we don't store the time unit we try to reconstuct it
     int unit = 0;
     if( value > 600 || !(value % 60) ) {
-        unit++;
+        unit = 1;
         value /= 60;
 
         if( value > 600 || !(value % 60) ) {
-            unit++;
+            unit = 2;
             value /= 60;
 
-            if( value > 600 || !(value % 24) ) {
-                unit++;
+            if( value > 72 || !(value % 24) ) {
+                unit = 3;
                 value /= 24;
 
-                if( value > 89 || !(value % 30) ) {
-                    unit++;
+                if( !(value % 365) ) {
+                    unit = 6;
+                    value /= 365;
+                } else if( !(value % 30) ) {
+                    unit = 5;
                     value /= 30;
+                } else if( !(value % 7) ) {
+                    unit = 4;
+                    value /= 7;
                 }
             }
         }
@@ -114,7 +128,7 @@ void TimeDistanceWidget::setTimeDistance( qint64 value )
 
 void TimeDistanceWidget::connectChanged( QObject *receiver, const char *slot )
 {
-    connect( m_timeEdit, SIGNAL(valueChanged(const QString&)), receiver, slot );
+    connect( m_timeEdit, SIGNAL(valueChanged(QString)), receiver, slot );
     connect( m_unitSelection, SIGNAL(currentIndexChanged(int)), receiver, slot );
 }
 
@@ -125,15 +139,110 @@ void TimeDistanceWidget::slotUpdateComboBoxLabels( int value )
     m_unitSelection->setItemText(1, i18np("minute", "minutes", value));
     m_unitSelection->setItemText(2, i18np("hour", "hours", value));
     m_unitSelection->setItemText(3, i18np("day", "days", value));
-    m_unitSelection->setItemText(4, i18np("month", "months", value));
+    m_unitSelection->setItemText(4, i18np("week", "weeks", value));
+    m_unitSelection->setItemText(5, i18np("month", "months", value));
+    m_unitSelection->setItemText(6, i18np("year", "years", value));
 }
 
+void
+MetaQueryWidget::Filter::setField( qint64 newField )
+{
+    if( m_field == newField )
+        return;
+
+    // -- reset the value and the condition if the new filter has another type
+    if( MetaQueryWidget::isNumeric( m_field ) != MetaQueryWidget::isNumeric( newField ) )
+    {
+        value.clear();
+        if( MetaQueryWidget::isNumeric( newField ) )
+            condition = Equals;
+        else
+            condition = Contains;
+    }
+    if( !MetaQueryWidget::isDate( m_field ) && MetaQueryWidget::isDate( newField ) )
+    {
+        numValue = QDateTime::currentDateTime().toTime_t();
+        numValue2 = QDateTime::currentDateTime().toTime_t();
+    }
+    else
+    {
+        numValue = 0;
+        numValue2 = 0;
+    }
+
+    if (numValue < minimumValue( newField ) || numValue > maximumValue( newField ) )
+        numValue = defaultValue( newField );
+
+    if (numValue2 < minimumValue( newField ) || numValue2 > maximumValue( newField ) )
+        numValue2 = defaultValue( newField );
+
+    m_field = newField;
+}
+
+qint64
+MetaQueryWidget::Filter::minimumValue( quint64 field )
+{
+    switch( field )
+    {
+    case Meta::valYear: return 1900;
+    case Meta::valTrackNr: return 0;
+    case Meta::valDiscNr: return 0;
+    case Meta::valBpm: return 60;
+    case Meta::valBitrate: return 60;
+    case Meta::valSamplerate: return 8000;
+    case Meta::valFilesize: return 0;
+    case Meta::valScore: return 0;
+    case Meta::valPlaycount: return 0;
+    case Meta::valRating: return 0;
+    case Meta::valLength: return 0;
+    default: return 0;
+    }
+}
+
+qint64
+MetaQueryWidget::Filter::maximumValue( quint64 field )
+{
+    switch( field )
+    {
+    case Meta::valYear: return 2300;
+    case Meta::valTrackNr: return 100;
+    case Meta::valDiscNr: return 10;
+    case Meta::valBpm: return 200;
+    case Meta::valBitrate: return 2000;
+    case Meta::valSamplerate: return 48000;
+    case Meta::valFilesize: return 1000;
+    case Meta::valScore: return 100;
+    case Meta::valPlaycount: return 1000;
+    case Meta::valRating: return 10;
+    case Meta::valLength: return maxHours * 60 * 60 - 1;
+    default: return 0;
+    }
+}
+
+qint64
+MetaQueryWidget::Filter::defaultValue( quint64 field )
+{
+    switch( field )
+    {
+    case Meta::valYear: return 1976;
+    case Meta::valTrackNr: return 0;
+    case Meta::valDiscNr: return 0;
+    case Meta::valBpm: return 80;
+    case Meta::valBitrate: return 160;
+    case Meta::valSamplerate: return 44100;
+    case Meta::valFilesize: return 10;
+    case Meta::valScore: return 0;
+    case Meta::valPlaycount: return 00;
+    case Meta::valRating: return 0;
+    case Meta::valLength: return 3 * 60 + 59;
+    default: return 0;
+    }
+}
 
 MetaQueryWidget::MetaQueryWidget( QWidget* parent, bool onlyNumeric, bool noCondition )
     : QWidget( parent )
     , m_onlyNumeric( onlyNumeric )
     , m_noCondition( noCondition )
-    , m_apgMode( false )
     , m_settingFilter( false )
     , m_andLabel(0)
     , m_compareSelection(0)
@@ -155,14 +264,10 @@ MetaQueryWidget::MetaQueryWidget( QWidget* parent, bool onlyNumeric, bool noCond
     m_layoutValueValues = new QVBoxLayout();
     m_layoutValue->addLayout(m_layoutValueValues, 1);
 
-    if( m_onlyNumeric ) {
-        m_filter.field = Meta::valYear;
-        m_filter.condition = Equals;
-
-    } else {
-        m_filter.field = 0;
-        m_filter.condition = Contains;
-    }
+    if( m_onlyNumeric )
+        m_filter.setField( Meta::valYear );
+    else
+        m_filter.setField( 0 );
 
     setFilter(m_filter);
 }
@@ -184,21 +289,13 @@ MetaQueryWidget::filter() const
     return m_filter;
 }
 
-void
-MetaQueryWidget::setAPGMode( bool value )
-{
-    m_apgMode = value;
-    makeValueSelection();
-    setValueSelection();
-}
-
 void
 MetaQueryWidget::setFilter( const MetaQueryWidget::Filter &value )
 {
     m_settingFilter = true;
     m_filter = value;
 
-    int index = m_fieldSelection->findData( int(m_filter.field) );
+    int index = m_fieldSelection->findData( int(m_filter.field()) );
     m_fieldSelection->setCurrentIndex( index == -1 ? 0 : index );
 
     if( !m_noCondition )
@@ -230,8 +327,8 @@ MetaQueryWidget::makeFieldSelection()
         addIconItem( m_fieldSelection, Meta::valUrl );
         // note: what about directory?
         addIconItem( m_fieldSelection, Meta::valTitle );
-        // note: album artist is not handled here. could be added
         addIconItem( m_fieldSelection, Meta::valArtist );
+        addIconItem( m_fieldSelection, Meta::valAlbumArtist );
         addIconItem( m_fieldSelection, Meta::valAlbum );
         addIconItem( m_fieldSelection, Meta::valGenre );
         addIconItem( m_fieldSelection, Meta::valComposer );
@@ -266,37 +363,13 @@ MetaQueryWidget::fieldChanged( int i )
     if( m_settingFilter )
         return;
 
-    qint64 field = Contains;
-    if( m_fieldSelection->count() == 0 )
-    {
-        if( isNumeric( field ) )
-            m_filter.condition = Equals;
-        else if( !isNumeric( field ) )
-            m_filter.condition = Contains;
-    }
-    else if( i<0 || i>=m_fieldSelection->count() )
+    qint64 field = 0;
+    if( i<0 || i>=m_fieldSelection->count() )
         field = m_fieldSelection->itemData( 0 ).toInt();
     else
         field = m_fieldSelection->itemData( i ).toInt();
 
-    if( m_filter.field == field )
-        return; // nothing to do
-
-    // -- if the field was changed, reset the values (but not always)
-    if( isNumeric( m_filter.field ) != isNumeric( field ) )
-        m_filter.value.clear();
-    if( !isDate( m_filter.field ) && isDate( field ) )
-    {
-        m_filter.numValue = QDateTime::currentDateTime().toTime_t();
-        m_filter.numValue2 = QDateTime::currentDateTime().toTime_t();
-    }
-    else
-    {
-        m_filter.numValue = 0;
-        m_filter.numValue2 = 0;
-    }
-
-    m_filter.field = field;
+    m_filter.setField( field );
 
     // in the fieldChanged slot we assume that the field was really changed,
     // so we don't have a problem with throwing away all the old widgets
@@ -318,6 +391,69 @@ MetaQueryWidget::compareChanged( int index )
     if( m_filter.condition == condition )
         return; // nothing to do
 
+    if( m_filter.isDate() )
+    {
+        if(  ( condition == OlderThan || condition == NewerThan )
+            && m_filter.condition != OlderThan && m_filter.condition != NewerThan
+          )
+        {
+            // fix some inaccuracies caused by the conversion absoulte/relative time specifications
+            // this is actually just for visual consistency
+            int unit = 0;
+            qint64 value = QDateTime::currentDateTime().toTime_t() - m_filter.numValue;
+            if( value > 600 || !(value % 60) ) {
+                unit = 1;
+                value /= 60;
+
+                if( value > 600 || !(value % 60) ) {
+                    unit = 2;
+                    value /= 60;
+
+                    if( value > 72 || !(value % 24) ) {
+                        unit = 3;
+                        value /= 24;
+
+                        if( !(value % 365) ) {
+                            unit = 6;
+                            value /= 365;
+                        } else if( !(value % 30) ) {
+                            unit = 5;
+                            value /= 30;
+                        } else if( !(value % 7) ) {
+                            unit = 4;
+                            value /= 7;
+                        }
+                    }
+                }
+            }
+            switch( unit )
+            {
+            case 6:
+                value *= 365*24*60*60; // years
+                break;
+            case 5:
+                value *=  30*24*60*60; // months
+                break;
+            case 4:
+                value *=   7*24*60*60; // weeks
+                break;
+            case 3:
+                value *= 24; // days
+            case 2:
+                value *= 60; // hours
+            case 1:
+                value *= 60; // minutes
+            }
+            m_filter.numValue = value;
+        }
+        else if( condition != OlderThan && condition != NewerThan
+            && ( m_filter.condition == OlderThan || m_filter.condition == NewerThan )
+          )
+        {
+            m_filter.numValue = QDateTime::currentDateTime().toTime_t() - m_filter.numValue;
+        }
+    }
+
     m_filter.condition = condition;
 
     // need to re-generate the value selection fields
@@ -415,11 +551,17 @@ MetaQueryWidget::numValue2DateChanged()
 void
 MetaQueryWidget::numValueTimeDistanceChanged()
 {
+    if( !sender() )
+        return;
+
     // static_cast. Remember: the TimeDistanceWidget does not have a Q_OBJECT macro
     TimeDistanceWidget* distanceSelection = static_cast<TimeDistanceWidget*>( sender()->parent() );
-    m_filter.numValue = distanceSelection->timeDistance();
+    if( distanceSelection )
+    {
+        m_filter.numValue = distanceSelection->timeDistance();
 
-    emit changed(m_filter);
+        emit changed(m_filter);
+    }
 }
 
 void
@@ -465,7 +607,7 @@ MetaQueryWidget::makeCompareSelection()
     delete m_compareSelection;
     m_compareSelection = 0;
 
-    qint64 field = m_filter.field;
+    qint64 field = m_filter.field();
 
     if( field == Meta::valFormat )
         return; // the field is fixed
@@ -473,36 +615,26 @@ MetaQueryWidget::makeCompareSelection()
     else if( isDate(field) )
     {
         m_compareSelection = new KComboBox();
-        m_compareSelection->addItem( conditionToString( Equals, true ), (int)Equals );
-        m_compareSelection->addItem( conditionToString( LessThan, true ), (int)LessThan );
-        m_compareSelection->addItem( conditionToString( GreaterThan, true ), (int)GreaterThan );
-        if( !m_apgMode )
-        {
-            m_compareSelection->addItem( conditionToString( Between, true ), (int)Between );
-            m_compareSelection->addItem( conditionToString( OlderThan, true ), (int)OlderThan );
-        }
-        else
-            m_compareSelection->addItem( conditionToString( Within, true ), (int)Within );
+        m_compareSelection->addItem( conditionToString( Equals, field ), (int)Equals );
+        m_compareSelection->addItem( conditionToString( LessThan, field ), (int)LessThan );
+        m_compareSelection->addItem( conditionToString( GreaterThan, field ), (int)GreaterThan );
+        m_compareSelection->addItem( conditionToString( Between, field ), (int)Between );
+        m_compareSelection->addItem( conditionToString( OlderThan, field ), (int)OlderThan );
+        m_compareSelection->addItem( conditionToString( NewerThan, field ), (int)NewerThan );
     }
     else if( isNumeric(field) )
     {
         m_compareSelection = new KComboBox();
-        m_compareSelection->addItem( conditionToString( Equals ), (int)Equals );
-        m_compareSelection->addItem( conditionToString( LessThan ), (int)LessThan );
-        m_compareSelection->addItem( conditionToString( GreaterThan ), (int)GreaterThan );
-        if( !m_apgMode )
-            m_compareSelection->addItem( conditionToString( Between ), (int)Between );
+        m_compareSelection->addItem( conditionToString( Equals, field ), (int)Equals );
+        m_compareSelection->addItem( conditionToString( LessThan, field ), (int)LessThan );
+        m_compareSelection->addItem( conditionToString( GreaterThan, field ), (int)GreaterThan );
+        m_compareSelection->addItem( conditionToString( Between, field ), (int)Between );
     }
     else
     {
-        if( !m_apgMode )
-            return; // no compare selection for text fields
-
         m_compareSelection = new KComboBox();
-        m_compareSelection->addItem( conditionToString( Contains ), (int)Contains );
-        m_compareSelection->addItem( conditionToString( Matches ), (int)Matches );
-        m_compareSelection->addItem( conditionToString( StartsWith ), (int)StartsWith );
-        m_compareSelection->addItem( conditionToString( EndsWith ), (int)EndsWith );
+        m_compareSelection->addItem( conditionToString( Contains, field ), (int)Contains );
+        m_compareSelection->addItem( conditionToString( Equals, field ), (int)Equals );
     }
 
     // -- select the correct entry (even if the condition is not one of the selection)
@@ -527,41 +659,42 @@ MetaQueryWidget::makeValueSelection()
     delete m_valueSelection2;
     m_valueSelection2 = 0;
 
-    qint64 field = m_filter.field;
+    qint64 field = m_filter.field();
     if( field == Meta::valUrl )
         makeFilenameSelection();
     else if( field == Meta::valTitle )
         // We,re not going to populate this. There tends to be too many titles.
         makeGenericComboSelection( true, 0 );
     else if( field == Meta::valArtist ||
+        field == Meta::valAlbumArtist ||
         field == Meta::valAlbum ||
         field == Meta::valGenre ||
         field == Meta::valComposer )
         makeMetaComboSelection( field );
     else if( field == Meta::valYear )
-        makeGenericNumberSelection( 1900, 2300, 1976 );
+        makeGenericNumberSelection( field );
     else if( field == Meta::valComment )
         makeGenericComboSelection( true, 0 );
     else if( field == Meta::valTrackNr )
-        makeGenericNumberSelection( 0, 100, 0 );
+        makeGenericNumberSelection( field );
     else if( field == Meta::valDiscNr )
-        makeGenericNumberSelection( 0, 10, 0 );
+        makeGenericNumberSelection( field );
     else if( field == Meta::valBpm )
-        makeGenericNumberSelection( 60, 200, 80 );
+        makeGenericNumberSelection( field );
     else if( field == Meta::valLength )
         makeLengthSelection();
     else if( field == Meta::valBitrate )
-        makeGenericNumberSelection( 60, 2000, 160, i18nc("Unit for data rate kilo bit per seconds", "kbps") );
+        makeGenericNumberSelection( field, i18nc("Unit for data rate kilo bit per seconds", "kbps") );
     else if( field == Meta::valSamplerate )
-        makeGenericNumberSelection( 8000, 48000, 44000, i18nc("Unit for sample rate", "Hz") );
+        makeGenericNumberSelection( field, i18nc("Unit for sample rate", "Hz") );
     else if( field == Meta::valFilesize )
-        makeGenericNumberSelection( 0, 1000, 10, i18nc("Unit for file size in mega byte", "MiB") );
+        makeGenericNumberSelection( field, i18nc("Unit for file size in mega byte", "MiB") );
     else if( field == Meta::valFormat )
         makeFormatComboSelection();
     else if( field == Meta::valCreateDate )
         makeDateTimeSelection();
     else if( field == Meta::valScore )
-        makeGenericNumberSelection( 0, 100, 0 );
+        makeGenericNumberSelection( field );
     else if( field == Meta::valRating )
         makeRatingSelection();
     else if( field == Meta::valFirstPlayed )
@@ -569,7 +702,7 @@ MetaQueryWidget::makeValueSelection()
     else if( field == Meta::valLastPlayed )
         makeDateTimeSelection();
     else if( field == Meta::valPlaycount )
-        makeGenericNumberSelection( 0, 1000, 0 );
+        makeGenericNumberSelection( field );
     else if( field == Meta::valLabel )
         makeGenericComboSelection( true, 0 );
     else if( field == Meta::valModified )
@@ -587,8 +720,8 @@ MetaQueryWidget::makeGenericComboSelection( bool editable, Collections::QueryMak
     if( populateQuery != 0 )
     {
         m_runningQueries.insert(populateQuery, QWeakPointer<KComboBox>(combo));
-        connect( populateQuery, SIGNAL(newResultReady(QString,QStringList)),
-                SLOT(populateComboBox(QString,QStringList)) );
+        connect( populateQuery, SIGNAL(newResultReady(QStringList)),
+                SLOT(populateComboBox(QStringList)) );
         connect( populateQuery, SIGNAL(queryDone()),
                 SLOT(comboBoxPopulated()) );
 
@@ -596,8 +729,8 @@ MetaQueryWidget::makeGenericComboSelection( bool editable, Collections::QueryMak
     }
     combo->setEditText( m_filter.value );
 
-    connect( combo, SIGNAL(editTextChanged( const QString& ) ),
-            SLOT(valueChanged(const QString&)) );
+    connect( combo, SIGNAL(editTextChanged(QString)),
+            SLOT(valueChanged(QString)) );
 
     combo->completionObject()->setIgnoreCase( true );
     combo->setCompletionMode( KGlobalSettings::CompletionPopup );
@@ -608,7 +741,7 @@ MetaQueryWidget::makeGenericComboSelection( bool editable, Collections::QueryMak
 void
 MetaQueryWidget::makeMetaComboSelection( qint64 field )
 {
-    Collections::QueryMaker* qm = new Collections::MetaQueryMaker( CollectionManager::instance()->queryableCollections() );
+    Collections::QueryMaker* qm = CollectionManager::instance()->queryMaker();
     qm->setQueryType( Collections::QueryMaker::Custom );
     qm->addReturnValue( field );
     qm->setAutoDelete( true );
@@ -616,10 +749,8 @@ MetaQueryWidget::makeMetaComboSelection( qint64 field )
 }
 
 void
-MetaQueryWidget::populateComboBox( QString collectionId, QStringList results )
+MetaQueryWidget::populateComboBox( QStringList results )
 {
-    Q_UNUSED(collectionId);
-
     QObject* query = sender();
     if( !query )
         return;
@@ -643,8 +774,8 @@ MetaQueryWidget::populateComboBox( QString collectionId, QStringList results )
 
     // reset the text and re-enable the signal
     combo.data()->setEditText( m_filter.value );
-    connect( combo.data(), SIGNAL(editTextChanged( const QString& ) ),
-            SLOT(valueChanged(const QString&)) );
+    connect( combo.data(), SIGNAL(editTextChanged(QString)),
+            SLOT(valueChanged(QString)) );
 }
 
 void
@@ -657,13 +788,13 @@ MetaQueryWidget::makeFormatComboSelection()
     {
         combo->addItem(filetypes.at(listpos),listpos);
     }
-    
+
     int index = m_fieldSelection->findData( (int)m_filter.numValue );
     combo->setCurrentIndex( index == -1 ? 0 : index );
 
     connect( combo,
-             SIGNAL( currentIndexChanged(int) ),
-             SLOT( numValueFormatChanged(int) ) );
+             SIGNAL(currentIndexChanged(int)),
+             SLOT(numValueFormatChanged(int)) );
 
     m_valueSelection1 = combo;
 }
@@ -692,7 +823,7 @@ MetaQueryWidget::makeRatingSelection()
     KRatingWidget* ratingWidget = new KRatingWidget();
     ratingWidget->setRating( (int)m_filter.numValue );
     connect( ratingWidget, SIGNAL(ratingChanged(int)),
-             this, SLOT( numValueChanged(int) ) );
+             this, SLOT(numValueChanged(int)) );
 
     m_valueSelection1 = ratingWidget;
 
@@ -712,14 +843,15 @@ MetaQueryWidget::makeRatingSelection()
 void
 MetaQueryWidget::makeLengthSelection()
 {
+    QString displayFormat = i18nc( "time format for specifying track length - hours, minutes, seconds", "h:m:ss" );
     QTimeEdit* timeSpin = new QTimeEdit();
-    timeSpin->setDisplayFormat( "m:ss" );
+    timeSpin->setDisplayFormat( displayFormat );
     timeSpin->setMinimumTime( QTime( 0, 0, 0 ) );
-    timeSpin->setMaximumTime( QTime( 0, 60, 59) );
+    timeSpin->setMaximumTime( QTime( maxHours - 1, 59, 59 ) );
     timeSpin->setTime( QTime().addSecs( m_filter.numValue ) );
 
-    connect( timeSpin, SIGNAL(timeChanged(const QTime&)),
-            SLOT(numValueChanged(const QTime&)) );
+    connect( timeSpin, SIGNAL(timeChanged(QTime)),
+            SLOT(numValueChanged(QTime)) );
 
     m_valueSelection1 = timeSpin;
 
@@ -727,33 +859,26 @@ MetaQueryWidget::makeLengthSelection()
         return;
 
     QTimeEdit* timeSpin2 = new QTimeEdit();
-    timeSpin2->setDisplayFormat( "m:ss" );
+    timeSpin2->setDisplayFormat( displayFormat );
     timeSpin2->setMinimumTime( QTime( 0, 0, 0 ) );
-    timeSpin2->setMaximumTime( QTime( 0, 60, 59) );
+    timeSpin2->setMaximumTime( QTime( maxHours - 1, 59, 59 ) );
     timeSpin2->setTime( QTime().addSecs( m_filter.numValue2 ) );
 
-    connect( timeSpin2, SIGNAL(timeChanged(const QTime&)),
-            SLOT(numValueChanged(const QTime&)) );
+    connect( timeSpin2, SIGNAL(timeChanged(QTime)),
+            SLOT(numValue2Changed(QTime)) );
 
     m_valueSelection2 = timeSpin2;
 }
 
 void
-MetaQueryWidget::makeGenericNumberSelection( int min, int max, int def, const QString& unit )
+MetaQueryWidget::makeGenericNumberSelection( qint64 field, const QString& unit )
 {
     KIntSpinBox* spin = new KIntSpinBox();
-    spin->setMinimum( min );
-    spin->setMaximum( max );
+    spin->setMinimum( Filter::minimumValue( field ) );
+    spin->setMaximum( Filter::maximumValue( field ) );
     if( !unit.isEmpty() )
-        spin->setSuffix( " " + unit );
-    if( m_filter.condition == Contains ||
-        m_filter.numValue < min || m_filter.numValue > max )
-    {
-        spin->setValue( def );
-        numValueChanged( def );
-    }
-    else
-        spin->setValue( m_filter.numValue );
+        spin->setSuffix( ' ' + unit );
+    spin->setValue( m_filter.numValue );
 
     connect( spin, SIGNAL(valueChanged(int)),
             this, SLOT(numValueChanged(int)) );
@@ -765,18 +890,11 @@ MetaQueryWidget::makeGenericNumberSelection( int min, int max, int def, const QS
 
     // second spin box for the between selection
     KIntSpinBox* spin2 = new KIntSpinBox();
-    spin2->setMinimum( min );
-    spin2->setMaximum( max );
+    spin2->setMinimum( Filter::minimumValue( field ) );
+    spin2->setMaximum( Filter::maximumValue( field ) );
     if( !unit.isEmpty() )
-        spin2->setSuffix( " " + unit );
-    if( m_filter.condition == Contains ||
-        m_filter.numValue2 < min || m_filter.numValue2 > max )
-    {
-        spin2->setValue( def );
-        numValue2Changed( def );
-    }
-    else
-        spin2->setValue( m_filter.numValue2 );
+        spin2->setSuffix( ' ' + unit );
+    spin2->setValue( m_filter.numValue2 );
 
     connect( spin2, SIGNAL(valueChanged(int)),
             this, SLOT(numValue2Changed(int)) );
@@ -788,19 +906,28 @@ MetaQueryWidget::makeGenericNumberSelection( int min, int max, int def, const QS
 void
 MetaQueryWidget::makeDateTimeSelection()
 {
-    if( m_filter.condition != OlderThan &&
-        m_filter.condition != Within )
+    if( m_filter.condition == OlderThan || m_filter.condition == NewerThan )
+    {
+        TimeDistanceWidget* distanceSelection = new TimeDistanceWidget();
+        distanceSelection->setTimeDistance( m_filter.numValue );
+
+        distanceSelection->connectChanged( this, SLOT(numValueTimeDistanceChanged()));
+
+        m_valueSelection1 = distanceSelection;
+    }
+    else
     {
         KDateCombo* dateSelection = new KDateCombo();
         QDateTime dt;
-        if( m_filter.condition == Contains )
-            dt = QDateTime::currentDateTime();
-        else
-            dt.setTime_t( m_filter.numValue );
+//         if( m_filter.condition == Contains || m_filter.condition == Equals )
+//             dt = QDateTime::currentDateTime();
+//         else
+//             dt.setTime_t( m_filter.numValue );
+        dt.setTime_t( m_filter.numValue );
         dateSelection->setDate( dt.date() );
 
         connect( dateSelection, SIGNAL(currentIndexChanged(int)),
-                SLOT( numValueDateChanged() ) );
+                SLOT(numValueDateChanged()) );
 
         m_valueSelection1 = dateSelection;
 
@@ -809,30 +936,19 @@ MetaQueryWidget::makeDateTimeSelection()
 
         // second KDateCombo for the between selection
         KDateCombo* dateSelection2 = new KDateCombo();
-        if( m_filter.condition == Contains )
-            dt = QDateTime::currentDateTime();
-        else
-            dt.setTime_t( m_filter.numValue2 );
+        dt.setTime_t( m_filter.numValue2 );
         dateSelection2->setDate( dt.date() );
 
         connect( dateSelection2, SIGNAL(currentIndexChanged(int)),
-                SLOT( numValue2DateChanged() ) );
+                SLOT(numValue2DateChanged()) );
 
         m_valueSelection2 = dateSelection2;
     }
-    else
-    {
-        TimeDistanceWidget* distanceSelection = new TimeDistanceWidget();
-        distanceSelection->setTimeDistance( m_filter.numValue);
-
-        distanceSelection->connectChanged( this, SLOT(numValueTimeDistanceChanged()));
-
-        m_valueSelection1 = distanceSelection;
-    }
 }
 
 
-bool MetaQueryWidget::isNumeric( qint64 field )
+bool
+MetaQueryWidget::isNumeric( qint64 field )
 {
     switch( field )
     {
@@ -858,7 +974,8 @@ bool MetaQueryWidget::isNumeric( qint64 field )
     }
 }
 
-bool MetaQueryWidget::isDate( qint64 field )
+bool
+MetaQueryWidget::isDate( qint64 field )
 {
     switch( field )
     {
@@ -872,9 +989,10 @@ bool MetaQueryWidget::isDate( qint64 field )
     }
 }
 
-QString MetaQueryWidget::conditionToString( FilterCondition condition, bool isDate )
+QString
+MetaQueryWidget::conditionToString( FilterCondition condition, qint64 field )
 {
-    if( isDate )
+    if( isDate(field) )
     {
         switch( condition )
         {
@@ -888,13 +1006,13 @@ QString MetaQueryWidget::conditionToString( FilterCondition condition, bool isDa
             return i18nc( "The date is between the given fixed dates", "between" );
         case OlderThan:
             return i18nc( "The date lies before the given time interval", "older than" );
-        case Within:
-            return i18nc( "The date lies after the given time interval", "within" );
+        case NewerThan:
+            return i18nc( "The date lies after the given time interval", "newer than" );
         default:
             ; // fall through
         }
     }
-    else
+    else if( isNumeric(field) )
     {
         switch( condition )
         {
@@ -906,14 +1024,18 @@ QString MetaQueryWidget::conditionToString( FilterCondition condition, bool isDa
             return i18n("greater than");
         case Between:
             return i18nc( "a numerical tag (like year or track number) is between two values", "between" );
+        default:
+            ; // fall through
+        }
+    }
+    else
+    {
+        switch( condition )
+        {
+        case Equals:
+            return i18nc("an alphabetical tag (like title or artist name) equals some string","equals");
         case Contains:
             return i18nc("an alphabetical tag (like title or artist name) contains some string", "contains");
-        case Matches:
-            return i18nc("an alphabetical tag (like title or artist name) equals some string","matches");
-        case StartsWith:
-            return i18nc("an alphabetical tag (like title or artist name) starts with some string","starts with");
-        case EndsWith:
-            return i18nc("an alphabetical tag (like title or artist name) ends with some string","ends with");
         default:
             ; // fall through
         }
@@ -921,47 +1043,68 @@ QString MetaQueryWidget::conditionToString( FilterCondition condition, bool isDa
     return QString( i18n("unknown comparison") );
 }
 
-QString MetaQueryWidget::Filter::fieldToString() const
+QString
+MetaQueryWidget::Filter::fieldToString() const
 {
-    return Meta::shortI18nForField( field );
+    return Meta::shortI18nForField( m_field );
 }
 
-QString MetaQueryWidget::Filter::toString( bool invert )
+QString MetaQueryWidget::Filter::toString( bool invert ) const
 {
     // this member is called when there is a keyword that needs numeric attributes
-    QString strField = fieldToString();
     QString strValue1 = value;
     QString strValue2 = value;
 
-    if( field == Meta::valFormat )
+    if( m_field == Meta::valFormat )
     {
         strValue1 = Amarok::FileTypeSupport::toString( Amarok::FileType( numValue ));
     }
-    else if( field == Meta::valRating )
+    else if( m_field == Meta::valRating )
     {
         strValue1 = QString::number( (float)numValue / 2 );
         strValue2 = QString::number( (float)numValue2 / 2 );
     }
-    else if( MetaQueryWidget::isDate(field) )
+    else if( isDate() )
     {
-        // here we are handling only the date. relative times are handled below
-        const QDateTime &today = KDateTime::currentLocalDateTime().dateTime();
-        strValue1 = QString::number( QDateTime::fromTime_t( numValue  ).daysTo( today )) + 'd';
-        strValue2 = QString::number( QDateTime::fromTime_t( numValue2 ).daysTo( today )) + 'd';
+        if( condition == OlderThan || condition == NewerThan )
+        {
+            strValue1 = QString::number( numValue  );
+            strValue2 = QString::number( numValue2 );
+        }
+        else
+        {
+            KLocalizedDate localizedDate1( QDateTime::fromTime_t(numValue).date() );
+            strValue1 = localizedDate1.formatDate( KLocale::ShortDate );
+            KLocalizedDate localizedDate2( QDateTime::fromTime_t(numValue2).date() );
+            strValue2 = localizedDate2.formatDate( KLocale::ShortDate );
+        }
     }
-    else if( MetaQueryWidget::isNumeric(field) )
+    else if( isNumeric() )
     {
-        strValue1 = QString::number( numValue );
-        strValue2 = QString::number( numValue2 );
+        if (numValue < numValue2) // two values are only used for "between". We want to order them by size
+        {
+            strValue1 = QString::number( numValue );
+            strValue2 = QString::number( numValue2 );
+        }
+        else
+        {
+            strValue1 = QString::number( numValue2 );
+            strValue2 = QString::number( numValue );
+        }
     }
 
     QString result;
+    if( m_field )
+        result = fieldToString() + ':';
 
     switch( condition )
     {
     case Equals:
         {
-            result = strField + ":" + strValue1;
+            if( isNumeric() )
+                result += strValue1;
+            else
+                result += '=' + QString( "\"%1\"" ).arg( value );
             if( invert )
                 result.prepend( QChar('-') );
             break;
@@ -969,7 +1112,7 @@ QString MetaQueryWidget::Filter::toString( bool invert )
 
     case GreaterThan:
         {
-            result = strField + ":>" + strValue1;
+            result += '>' + strValue1;
             if( invert )
                 result.prepend( QChar('-') );
             break;
@@ -977,7 +1120,7 @@ QString MetaQueryWidget::Filter::toString( bool invert )
 
     case LessThan:
         {
-            result = strField + ":<" + strValue1;
+            result +='<' + strValue1;
             if( invert )
                 result.prepend( QChar('-') );
             break;
@@ -986,44 +1129,48 @@ QString MetaQueryWidget::Filter::toString( bool invert )
     case Between:
         {
             if( invert )
-                result = strField + ":<" + strValue1 + " OR " + strField + ":>" + strValue2;
+                result = QString( "%1<%2 OR %1>%3" ).arg( result, strValue1, strValue2 );
             else
-                result = strField + ":>" + strValue1 + " " + strField + ":<" + strValue2;
+                result = QString( "%1>%2 AND %1<%3" ).arg( result, strValue1, strValue2 );
             break;
         }
 
     case OlderThan:
-    case Within:
+    case NewerThan:
         {
             // a human readable time..
-            int unit = 0;
-            qint64 val = numValue;
-            if( val > 600 || !(val % 60) ) {
-                unit++;
-                val /= 60;
+            QChar strUnit = 's';
+            qint64 value = numValue;
+            if( !(value % 60) ) {
+                strUnit = 'M';
+                value /= 60;
 
-                if( val > 600 || !(val % 60) ) {
-                    unit++;
-                    val /= 60;
+                if( !(value % 60) ) {
+                    strUnit = 'h';
+                    value /= 60;
 
-                    if( val > 600 || !(val % 24) ) {
-                        unit++;
-                        val /= 24;
+                    if( !(value % 24) ) {
+                        strUnit = 'd';
+                        value /= 24;
+
+                        if( !(value % 365) ) {
+                            strUnit = 'y';
+                            value /= 365;
+                        } else if( !(value % 30) ) {
+                            strUnit = 'm';
+                            value /= 30;
+                        } else if( !(value % 7) ) {
+                            strUnit = 'w';
+                            value /= 7;
+                        }
                     }
                 }
             }
-            QChar strUnit('s');
-            if( unit==1 )
-                strUnit = 'M';
-            else if( unit==2 )
-                strUnit = 'h';
-            else if( unit==3 )
-                strUnit = 'd';
 
             if( condition == OlderThan )
-                result = strField + ":>" + QString::number(val) + strUnit;
+                result += '>' + QString::number(value) + strUnit;
             else
-                result = strField + ":<" + QString::number(val) + strUnit;
+                result += '<' + QString::number(value) + strUnit;
             if( invert )
                 result.prepend( QChar('-') );
             break;
@@ -1031,35 +1178,34 @@ QString MetaQueryWidget::Filter::toString( bool invert )
 
     case Contains:
         {
-            result = value;
-            if( invert )
-                result.prepend( QChar('-') );
-            break;
-        }
-    case Matches:
-        {
-            result = '"'+value+'"';
-            if( invert )
-                result.prepend( QChar('-') );
-            break;
-        }
-    case StartsWith:
-        {
-            result = '"'+value+"*\"";
-            if( invert )
-                result.prepend( QChar('-') );
-            break;
-        }
-    case EndsWith:
-        {
-            result = "\"*"+value+'"';
+            result += QString( "\"%1\"" ).arg( value );
             if( invert )
                 result.prepend( QChar('-') );
             break;
         }
     }
+
     return result;
 }
 
+bool
+MetaQueryWidget::isFieldSelectorHidden() const
+{
+    return m_fieldSelection->isHidden();
+}
+
+void
+MetaQueryWidget::setFieldSelectorHidden( const bool hidden )
+{
+    m_fieldSelection->setVisible( !hidden );
+}
+
+void
+MetaQueryWidget::setField( const qint64 field )
+{
+    int index = m_fieldSelection->findData( field );
+    m_fieldSelection->setCurrentIndex( index == -1 ? 0 : index );
+}
+
 #include "MetaQueryWidget.moc"
 
