@@ -16,14 +16,14 @@
 
 #include "TestMasterSlaveSynchronizationJob.h"
 
-#include "Debug.h"
-#include "collection/CollectionLocation.h"
-#include "collection/CollectionLocationDelegate.h"
-#include "Components.h"
+#include "core/support/Debug.h"
+#include "core/collections/CollectionLocation.h"
+#include "core/collections/CollectionLocationDelegate.h"
+#include "core/support/Components.h"
 #include "synchronization/MasterSlaveSynchronizationJob.h"
 
 #include "CollectionTestImpl.h"
-#include "collection/MockCollectionLocationDelegate.h"
+#include "core/collections/MockCollectionLocationDelegate.h"
 #include "mocks/MockTrack.h"
 #include "mocks/MockAlbum.h"
 #include "mocks/MockArtist.h"
@@ -44,10 +44,12 @@ using ::testing::_;
 static int trackCopyCount;
 static int trackRemoveCount;
 
+namespace Collections {
+
 class MyCollectionLocation : public CollectionLocation
 {
 public:
-    CollectionTestImpl *coll;
+    Collections::CollectionTestImpl *coll;
 
     QString prettyLocation() const { return "foo"; }
     bool isWritable() const { return true; }
@@ -64,8 +66,9 @@ public:
         slotRemoveOperationFinished();
     }
 
-    void copyUrlsToCollection(const QMap<Meta::TrackPtr, KUrl> &sources)
+    void copyUrlsToCollection(const QMap<Meta::TrackPtr, KUrl> &sources, const Transcoding::Configuration& conf)
     {
+        Q_UNUSED( conf )
         trackCopyCount = sources.count();
         foreach( const Meta::TrackPtr &track, sources.keys() )
         {
@@ -79,26 +82,27 @@ class MyCollectionTestImpl : public CollectionTestImpl
 public:
     MyCollectionTestImpl( const QString &id ) : CollectionTestImpl( id ) {}
 
-    CollectionLocation* location() const
+    CollectionLocation* location()
     {
         MyCollectionLocation *r = new MyCollectionLocation();
-        r->coll = const_cast<MyCollectionTestImpl*>( this );
+        r->coll = this;
         return r;
     }
 };
 
-void addMockTrack( CollectionTestImpl *coll, const QString &trackName, const QString &artistName, const QString &albumName )
+} //namespace Collections
+
+void addMockTrack( Collections::CollectionTestImpl *coll, const QString &trackName, const QString &artistName, const QString &albumName )
 {
     Meta::MockTrack *track = new Meta::MockTrack();
     ::testing::Mock::AllowLeak( track );
     Meta::TrackPtr trackPtr( track );
     EXPECT_CALL( *track, name() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName ) );
-    EXPECT_CALL( *track, uidUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName + "_" + artistName + "_" + albumName ) );
-    EXPECT_CALL( *track, isPlayable() ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
+    EXPECT_CALL( *track, uidUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName + '_' + artistName + '_' + albumName ) );
     EXPECT_CALL( *track, playableUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( KUrl( '/' + track->uidUrl() ) ) );
     coll->mc->addTrack( trackPtr );
 
-    Meta::AlbumPtr albumPtr = coll->mc->albumMap().value( albumName );
+    Meta::AlbumPtr albumPtr = coll->mc->albumMap().value( albumName, QString() /* no album artist */ );
     Meta::MockAlbum *album;
     Meta::TrackList albumTracks;
     if( albumPtr )
@@ -118,6 +122,7 @@ void addMockTrack( CollectionTestImpl *coll, const QString &trackName, const QSt
         albumPtr = Meta::AlbumPtr( album );
         EXPECT_CALL( *album, name() ).Times( AnyNumber() ).WillRepeatedly( Return( albumName ) );
         EXPECT_CALL( *album, hasAlbumArtist() ).Times( AnyNumber() ).WillRepeatedly( Return( false ) );
+        EXPECT_CALL( *album, albumArtist() ).Times( AnyNumber() ).WillRepeatedly( Return( Meta::ArtistPtr() ) );
         coll->mc->addAlbum( albumPtr );
     }
     albumTracks << trackPtr;
@@ -170,8 +175,8 @@ TestMasterSlaveSynchronizationJob::init()
 void
 TestMasterSlaveSynchronizationJob::testAddTracksToEmptySlave()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
     //setup master
     addMockTrack( master, "track1", "artist1", "album1" );
@@ -197,8 +202,8 @@ TestMasterSlaveSynchronizationJob::testAddTracksToEmptySlave()
 void
 TestMasterSlaveSynchronizationJob::testAddSingleTrack()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
     //setup
     addMockTrack( master, "track1", "artist1", "album1" );
@@ -230,8 +235,8 @@ TestMasterSlaveSynchronizationJob::testAddSingleTrack()
 void
 TestMasterSlaveSynchronizationJob::testAddAlbum()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
     //setup
     addMockTrack( master, "track1", "artist1", "album1" );
@@ -263,8 +268,8 @@ TestMasterSlaveSynchronizationJob::testAddAlbum()
 void
 TestMasterSlaveSynchronizationJob::testAddArtist()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
     //setup
     addMockTrack( master, "track1", "artist1", "album1" );
@@ -296,10 +301,10 @@ TestMasterSlaveSynchronizationJob::testAddArtist()
 void
 TestMasterSlaveSynchronizationJob::testRemoveSingleTrack()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
-    MockCollectionLocationDelegate *delegate = new MockCollectionLocationDelegate();
+    Collections::MockCollectionLocationDelegate *delegate = new Collections::MockCollectionLocationDelegate();
     EXPECT_CALL( *delegate, reallyDelete( _, _) ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
     Amarok::Components::setCollectionLocationDelegate( delegate );
 
@@ -334,10 +339,10 @@ TestMasterSlaveSynchronizationJob::testRemoveSingleTrack()
 void
 TestMasterSlaveSynchronizationJob::testRemoveAlbum()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
-    MockCollectionLocationDelegate *delegate = new MockCollectionLocationDelegate();
+    Collections::MockCollectionLocationDelegate *delegate = new Collections::MockCollectionLocationDelegate();
     EXPECT_CALL( *delegate, reallyDelete( _, _) ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
     Amarok::Components::setCollectionLocationDelegate( delegate );
 
@@ -372,10 +377,10 @@ TestMasterSlaveSynchronizationJob::testRemoveAlbum()
 void
 TestMasterSlaveSynchronizationJob::testRemoveArtist()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
-    MockCollectionLocationDelegate *delegate = new MockCollectionLocationDelegate();
+    Collections::MockCollectionLocationDelegate *delegate = new Collections::MockCollectionLocationDelegate();
     EXPECT_CALL( *delegate, reallyDelete( _, _) ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
     Amarok::Components::setCollectionLocationDelegate( delegate );
 
@@ -410,10 +415,10 @@ TestMasterSlaveSynchronizationJob::testRemoveArtist()
 void
 TestMasterSlaveSynchronizationJob::testEmptyMaster()
 {
-    CollectionTestImpl *master = new MyCollectionTestImpl( "master" );
-    CollectionTestImpl *slave = new MyCollectionTestImpl( "slave" );
+    Collections::CollectionTestImpl *master = new Collections::MyCollectionTestImpl( "master" );
+    Collections::CollectionTestImpl *slave = new Collections::MyCollectionTestImpl( "slave" );
 
-    MockCollectionLocationDelegate *delegate = new MockCollectionLocationDelegate();
+    Collections::MockCollectionLocationDelegate *delegate = new Collections::MockCollectionLocationDelegate();
     EXPECT_CALL( *delegate, reallyDelete( _, _) ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
     Amarok::Components::setCollectionLocationDelegate( delegate );
 
